/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.axis.example.weather;

import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPMessage;

import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.axis.AxisConfiguration;
import org.ow2.orchestra.axis.test.AxisTestCase;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.util.SOAPUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

/**
 * @author Charles Souillard
 */
public class WeatherTest extends AxisTestCase {

  public void testWeather() {
    try {
      String processName = "weather";
      String processNamespace = "http://orchestra.ow2.org/weather";
      QName processQName = new QName(processNamespace, processName);

      deploy(this.getClass().getResource(processName + ".bpel"), this
          .getClass().getResource(processName + "Artifacts.wsdl"));

      AxisConfiguration config = AxisConfiguration.getInstance();
      String endpoint = "http://" + config.getHost() + ":" + config.getPort()
          + "/" + config.getWebappName() + "/" + config.getServletPath() + "/"
          + "weatherPT";

      SOAPMessage request = SOAPUtil
          .buildDocumentSOAPMessage(
              "http://orchestra.ow2.org/weatherArtifacts/process",
              "<weatherRequest xmlns='http://orchestra.ow2.org/weather'><input>Grenoble,France</input></weatherRequest>");

      SOAPMessage response = SOAPUtil.call(request, endpoint);

      if (response != null) {
        SOAPBodyElement result = (SOAPBodyElement) response.getSOAPBody()
            .getChildElements().next();

        assertNotNull(result);
        assertEquals(XmlUtil.toString(result), "weatherResponse", result.getLocalName());
        assertEquals(processNamespace, result.getNamespaceURI());
        List<Element> children = BpelUtil.getElements(result);
        assertEquals(1, children.size());
        assertEquals("result", children.get(0).getLocalName());
        assertEquals(processNamespace, children.get(0).getNamespaceURI());

        List<Text> texts = BpelUtil.getTexts(children.get(0));
        assertEquals(1, texts.size());
        String s = texts.get(0).getData();

        Document doc = BpelUtil.getDocumentFromString(s);
        assertNotNull(doc);
        Element rootElement = doc.getDocumentElement();
        assertNotNull(rootElement);

        assertEquals("CurrentWeather", rootElement.getTagName());
        List<Element> elements = BpelUtil.getElements(rootElement);
        assertTrue(elements.size() >= 8);
        Element location = (Element) rootElement.getElementsByTagName(
            "Location").item(0);
        assertEquals(
            "Grenoble / St. Geoirs, France (LFLS) 45-22N 005-20E 386M",
            location.getTextContent());

        /*
         * <?xml version='1.0' encoding='utf-16'?> <CurrentWeather>
         * <Location>Grenoble / St. Geoirs, France (LFLS) 45-22N 005-20E 386M</Location>
         * <Time>Oct 08, 2007 - 09:00 AM EDT / 2007.10.08 1300 UTC</Time>
         * <Wind> from the NNW (340 degrees) at 10 MPH (9 KT) (direction
         * variable):0</Wind> <Visibility> greater than 7 mile(s):0</Visibility>
         * <Temperature> 62 F (17 C)</Temperature> <DewPoint> 51 F (11 C)</DewPoint>
         * <RelativeHumidity> 67%</RelativeHumidity> <Pressure> 30.18 in. Hg
         * (1022 hPa)</Pressure> <Status>Success</Status> </CurrentWeather>
         */
      } else {
        assertTrue("No return available", false);
      }

      undeploy(processQName);

    } catch (Exception e) {
      throw new OrchestraException(e);
    }
  }
}
