/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.axis.test;

import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.LogManager;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPMessage;

import junit.framework.TestCase;

import org.ow2.orchestra.axis.AxisConfiguration;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.jmx.InstanceInfo;
import org.ow2.orchestra.jmx.JMXConfiguration;
import org.ow2.orchestra.jmx.ProcessInfo;
import org.ow2.orchestra.jmxclient.DeployAction;
import org.ow2.orchestra.jmxclient.GetProcessAction;
import org.ow2.orchestra.jmxclient.ListInstanceAction;
import org.ow2.orchestra.jmxclient.ListProcessAction;
import org.ow2.orchestra.jmxclient.UndeployAction;
import org.ow2.orchestra.test.BpelTestCase;

public class AxisTestCase extends TestCase {

  static {
    try {
      InputStream inputStream = BpelTestCase.class.getClassLoader()
          .getResourceAsStream("logging.properties");
      if (inputStream != null) {
        LogManager.getLogManager().readConfiguration(inputStream);
      }
    } catch (Exception e) {
      throw new RuntimeException("couldn't initialize logging properly", e);
    }
  }

  protected String jmxServiceUrl = JMXConfiguration.getInstance()
      .getServiceUrl();

  protected String jmxObjectName = JMXConfiguration.getInstance()
      .getObjectName();

  protected void deploy(String processName) {
    deploy(processName, null);
  }

  protected void deploy(String processName, List<URL> wsdls) {
    deploy(this.getClass().getResource(processName + ".bpel"), wsdls);
  }

  protected void deploy(URL bpel, URL wsdl) {
    List<URL> wsdls = new ArrayList<URL>();
    wsdls.add(wsdl);
    deploy(bpel, wsdls);
  }

  protected void deploy(URL bpel, List<URL> wsdls) {
    assertNotNull(DeployAction
        .deploy(bpel, wsdls, jmxServiceUrl, jmxObjectName));
  }

  protected void getProcess(QName processQName) {
    ProcessInfo processInfo = GetProcessAction.get(jmxServiceUrl, jmxObjectName, processQName);
    //for debug
    System.out.println("*** process info: " + processInfo.getQname());
  }

  protected void listInstance() {
    Map<QName, List<InstanceInfo>> instances = null;
    List<QName> processNames = ListProcessAction.list(jmxServiceUrl,
        jmxObjectName);

    instances = ListInstanceAction.list(jmxServiceUrl, jmxObjectName);
    //for debug
    System.out.println("List of running instances per process:");
    for (QName qName : processNames) {
      System.out.println("Process qname = " + qName);
      List<InstanceInfo> insts = instances.get(qName);
      for (InstanceInfo instRec : insts) {
        System.out.println("instance uid = " + instRec.getInstanceUId() + " nodeName ="
            + instRec.getNodeName() + "= " + " state = " + instRec.getState() );

      }
    }
  }

  protected void undeploy(QName processQName) {
    assertTrue(UndeployAction.undeploy(processQName, jmxServiceUrl,
        jmxObjectName));
  }



  protected SOAPBodyElement getSOAPBodyElement(SOAPMessage response) {
    if (response == null) {
      throw new OrchestraException("response is null");
    }
    try {
      SOAPBody soapBody = response.getSOAPBody();
      if (soapBody == null) {
        throw new OrchestraException("response.soapBody is null");
      }
      Iterator<SOAPBodyElement> it = (Iterator<SOAPBodyElement>) soapBody
          .getChildElements();
      if (!it.hasNext()) {
        throw new OrchestraException("response.soapBody.children is empty");
      }
      SOAPBodyElement result = it.next();
      if (it.hasNext()) {
        throw new OrchestraException(
            "response.soapBody.children has more than one child");
      }
      return result;
    } catch (Exception e) {
      throw new OrchestraException(
          "Exception caught while getting soapBodyElement of message : "
              + response, e);
    }
  }

  protected String getDefaultEndpoint(String portName) {
    AxisConfiguration config = AxisConfiguration.getInstance();
    String endpoint = "http://" + config.getHost() + ":" + config.getPort()
        + "/" + config.getWebappName() + "/" + config.getServletPath() + "/"
        + portName;

    return endpoint;
  }
}
