/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.axis.test.activities.receive;

import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPMessage;

import junit.framework.AssertionFailedError;

import org.ow2.orchestra.axis.test.AxisTestCase;
import org.ow2.orchestra.util.SOAPUtil;
import org.w3c.dom.Text;

/**
 * @author Charles Souillard
 */
public class Receive2Test extends AxisTestCase {

  private static final String SOAPACTION1 = "http://orchestra.ow2.org/tests/receiveSeq/receive";

  private static final String SOAPACTION2 = "http://orchestra.ow2.org/tests/receiveSeq/receive2";

  public void testReceive2() {
    try {
      String processName = "receive2";
      String processNamespace = "http://example.com/receive2";
      QName processQName = new QName(processNamespace, processName);
      URL bpelURL = org.ow2.orchestra.test.activities.receive.Receive2Test.class
          .getResource(processName + ".bpel");
      URL wsdlURL = org.ow2.orchestra.test.activities.receive.Receive2Test.class
          .getResource(processName + ".wsdl");
      deploy(bpelURL, wsdlURL);

      ReceiveThread rt1 = new ReceiveThread("op1", "receive1", SOAPACTION1);
      ReceiveThread rt2 = new ReceiveThread("op2", "receive2", SOAPACTION2);

      Thread t1 = new Thread(rt1);
      Thread t2 = new Thread(rt2);

      t1.start();
      t2.start();

      // Wait for answers
      while (t1.isAlive() || t2.isAlive()) {
        t1.join(100);
        t2.join(100);

        if (rt1.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread").initCause(rt1
              .getTrowable());
        }
        if (rt2.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread").initCause(rt2
              .getTrowable());
        }
      }
      undeploy(processQName);
    } catch (Throwable e) {
      if (e instanceof AssertionFailedError) {
        throw (AssertionFailedError) e;
      }
      e.printStackTrace();
      assertTrue(false);
    }
  }

  class ReceiveThread implements Runnable {

    private Throwable uncaughtException;

    private String operation;

    private String partValue;

    private String soapAction;

    public Throwable getTrowable() {
      return uncaughtException;
    }

    public ReceiveThread(String operation, String partValue, String soapAction) {
      this.operation = operation;
      this.partValue = partValue;
      this.soapAction = soapAction;
    }

    public void run() {
      try {
        String endpoint = getDefaultEndpoint("receive2Port");
        SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(soapAction, "<"
            + operation + ">" + partValue + "</" + operation + ">");
        SOAPMessage response = SOAPUtil.call(request, endpoint);

        if (response == null) {
          assertTrue("No response available for receiveThread : " + operation,
              false);
        } else {
          SOAPBodyElement result = getSOAPBodyElement(response);
          checkReturn(result, partValue);
        }

      } catch (Throwable e) {
        uncaughtException = e;
      }
    }
  }

  private void checkReturn(SOAPBodyElement result, String expectedValue) {
    assertNotNull(result);
    assertEquals("response", result.getElementName().getLocalName());
    assertNotNull(result.getFirstChild());
    assertTrue(result.getFirstChild() instanceof Text);
    Text text = (Text) result.getFirstChild();
    assertEquals(expectedValue, text.getData());
  }

}
