/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.axis.test.factorial;

import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPMessage;

import org.ow2.orchestra.axis.AxisConfiguration;
import org.ow2.orchestra.axis.test.AxisTestCase;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.util.SOAPUtil;
import org.w3c.dom.Text;

/**
 * @author Charles Souillard
 */
public class FactorialTest extends AxisTestCase {

  /**
   * test.
   */
  public void testFactorial() {
    try {
      String processName = "factorial";
      String processNamespace = "http://orchestra.ow2.org/factorial";
      QName processQName = new QName(processNamespace, processName);
      deploy(processName);

      AxisConfiguration config = AxisConfiguration.getInstance();
      String endpoint = "http://" + config.getHost() + ":" + config.getPort()
          + "/" + config.getWebappName() + "/" + config.getServletPath() + "/"
          + "factorialPort";

      int nb = 6;
      SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(
          "http://orchestra.ow2.org/factorial",
          "<factorial xmlns='http://orchestra.ow2.org/factorial'>"
              + Integer.toString(nb) + "</factorial>");

      SOAPMessage response = SOAPUtil.call(request, endpoint);
      if (response != null) {
        SOAPBodyElement result = (SOAPBodyElement) response.getSOAPBody()
            .getChildElements().next();
        assertNotNull(result);
        assertEquals("factorial", result.getLocalName());
        assertEquals(processNamespace, result.getNamespaceURI());
        List<Text> texts = BpelUtil.getTexts(result);
        assertEquals(1, texts.size());
        assertEquals(Integer.toString(getFactorial(nb)), texts.get(0).getData());
      } else {
        assertTrue("No return available", false);
      }

      undeploy(processQName);

    } catch (Exception e) {
      throw new OrchestraException(e);
    }
  }

  public int getFactorial(int n) {
    if (n == 1) {
      return 1;
    } else {
      return n * getFactorial(n - 1);
    }
  }
}
