/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.axis.test.hello2OneBinding;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPMessage;

import junit.framework.AssertionFailedError;

import org.ow2.orchestra.axis.AxisConfiguration;
import org.ow2.orchestra.axis.test.AxisTestCase;
import org.ow2.orchestra.util.SOAPUtil;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

/**
 *
 * Tests a hello2 process: - 1 service to deploy - 2 receive/reply
 * 2 operations are on the same port / same binding
 * This causes an error
 * @see org.ow2.orchestra.axis.test.hello2
 * <br />There is no error if the bindings are distinct
 *
 * @author Pascal Verdage
 */
public class Hello2OneServiceTest extends AxisTestCase {

  static class Hello2CallHelloPT implements Runnable {

    private Throwable uncaughtException;

    public Throwable getTrowable() {
      return uncaughtException;
    }

    public void run() {
      try {
        String part = "<name>toto</name>";
        SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(
            "http://orchestra.ow2.org/hello2OneService/sayHello", part);
        AxisConfiguration config = AxisConfiguration.getInstance();
        String endpoint = "http://" + config.getHost() + ":" + config.getPort()
            + "/" + config.getWebappName() + "/" + config.getServletPath()
            + "/" + "helloPT";
        SOAPMessage response = SOAPUtil.call(request, endpoint);
        if (response != null) {
          SOAPBodyElement result = (SOAPBodyElement) response.getSOAPBody()
              .getChildElements().next();
          assertNotNull(result);
          // assertEquals("result", result.getElementName().getLocalName());
          assertNotNull(result.getFirstChild());
          assertTrue("Fail here because there are two operations in the same binding", result.getFirstChild() instanceof Text);
          Text text = (Text) result.getFirstChild();
          assertEquals("coucou toto", text.getData());
        } else {
          assertTrue("No return available", false);
        }
      } catch (Throwable e) {
        uncaughtException = e;
      }
    }
  }

  static class Hello2CallHelloTextPT implements Runnable {

    private Throwable uncaughtException;

    public Throwable getTrowable() {
      return uncaughtException;
    }

    public void run() {
      try {
        String part = "<setHelloText>coucou </setHelloText>";
        SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(
            "http://orchestra.ow2.org/hello2OneService/setHelloText", part);
        AxisConfiguration config = AxisConfiguration.getInstance();
        String endpoint = "http://" + config.getHost() + ":" + config.getPort()
            + "/" + config.getWebappName() + "/" + config.getServletPath()
            + "/" + "helloTextPT";
        SOAPMessage response = SOAPUtil.call(request, endpoint);
        if (response != null) {
          SOAPBodyElement result = (SOAPBodyElement) response.getSOAPBody()
              .getChildElements().next();
          assertNotNull(result);
          assertEquals(result.toString(), "setHelloText", result
              .getElementName().getLocalName());
          assertNotNull(result.getFirstChild());
          assertTrue(result.getFirstChild() instanceof Text);
          Text text = (Text) result.getFirstChild();
          assertEquals("coucou", text.getData());
        } else {
          assertTrue("No return available", false);
        }
      } catch (Throwable e) {
        uncaughtException = e;
      }
    }
  }

  /**
   * test.
   */
  public void testAxisHello2() {
    try {
      String processName = "hello2OneService";
      String processNamespace = "http://orchestra.ow2.org/demos/hello2OneService";
      QName processQName = new QName(processNamespace, processName);

      deploy(processName);

      Hello2CallHelloTextPT helloTextCall = new Hello2CallHelloTextPT();
      Hello2CallHelloPT helloCall = new Hello2CallHelloPT();

      Thread t1 = new Thread(helloTextCall);
      Thread t2 = new Thread(helloCall);

      // Do the first call
      t1.start();
      // Do the second call
      t2.start();

      // Wait for answers
      while (t1.isAlive() || t2.isAlive()) {
        t1.join(100);
        t2.join(100);
        if (helloCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread").initCause(helloCall
              .getTrowable());
        }
        if (helloTextCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread")
              .initCause(helloTextCall.getTrowable());
        }
      }

      // undeploy services
      undeploy(processQName);

    } catch (Throwable e) {
      if (e instanceof AssertionFailedError) {
        throw (AssertionFailedError) e;
      }
      e.printStackTrace();
      assertTrue(false);
    }
  }

}
