/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.axis.test.helloworld;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPMessage;

import org.ow2.orchestra.axis.AxisConfiguration;
import org.ow2.orchestra.axis.test.AxisTestCase;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.util.SOAPUtil;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

/**
 * @author Charles Souillard
 */
public class HelloworldTest extends AxisTestCase {

  /**
   * test.
   */
  public void testAxisHelloworld() {
    try {
      String processName = "helloworld";
      String processNamespace = "http://example.com/helloworld";
      QName processQName = new QName(processNamespace, processName);

      deploy(processName);

      String part = "<submit><st>Dupont</st></submit>";
      SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(
          "http://orchestra.ow2.org/helloworld/submit", part);

      AxisConfiguration config = AxisConfiguration.getInstance();
      String endpoint = "http://" + config.getHost() + ":" + config.getPort()
          + "/" + config.getWebappName() + "/" + config.getServletPath() + "/"
          + "helloworldPort";

      SOAPMessage response = SOAPUtil.call(request, endpoint);
      if (response != null) {
        SOAPBodyElement result = (SOAPBodyElement) response.getSOAPBody()
            .getChildElements().next();
        checkReturn(result, "Dupont");
      } else {
        assertTrue("No return available", false);
      }

      part = "<submit><st>Toto</st></submit>";
      request = SOAPUtil.buildDocumentSOAPMessage(
          "http://orchestra.ow2.org/helloworld/submit", part);
      response = SOAPUtil.call(request, endpoint);
      if (response != null) {
        SOAPBodyElement result = (SOAPBodyElement) response.getSOAPBody()
            .getChildElements().next();
        checkReturn(result, "Toto");
      } else {
        assertTrue("No return available", false);
      }
      undeploy(processQName);
    } catch (Exception e) {
      throw new OrchestraException(e);
    }
  }

  private void checkReturn(SOAPBodyElement result, String name) {
    assertNotNull(result);
    assertEquals("submit", result.getElementName().getLocalName());
    assertNotNull(result.getFirstChild());
    assertTrue(result.getFirstChild() instanceof Element);
    Element st = (Element) result.getFirstChild();
    assertEquals("st", st.getLocalName());
    assertNotNull(st.getFirstChild());
    assertTrue(st.getFirstChild() instanceof Text);
    Text text = (Text) st.getFirstChild();
    assertEquals(name, text.getData());
  }

}
