/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.axis.test.perf;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import org.ow2.novabpm.perf.StressPerfTest;


public class AxisStressTest extends org.ow2.orchestra.test.StressTest {

  public static void main(String[] args) throws Exception {
    Map<String, String> aliasesMap = new TreeMap<String, String>();
    Map<String, String[]> globalAliasesMap = new HashMap<String, String[]>();

    aliasesMap.putAll(getClassAliasesMap());
    aliasesMap.putAll(getAxisAliasesMap());

    globalAliasesMap.putAll(getClassGlobalAliasesMap());
    globalAliasesMap.putAll(getAxisGlobalAliasesMap());
    globalAliasesMap.put("all", (String[]) aliasesMap.keySet().toArray(
        new String[] {}));

    StressPerfTest test = new StressPerfTest(args, aliasesMap, globalAliasesMap);

    test.deployTests();
    test.launchTests();
  }

  public static void testStress() throws Exception {
    String[] myArgs = new String[10];
    myArgs[0] = "threadNb=10";
    myArgs[1] = "printFinished=false";
    myArgs[2] = "printLaunched=false";
    myArgs[3] = "timeBetweenVerifications=2s";
    myArgs[4] = "timeBetweenPrints=10s";
    myArgs[5] = "testsToRunClasses=axis.all";
    myArgs[6] = "thinkTime=10ms";
    myArgs[7] = "loadTime=1m";
    myArgs[8] = "warmupTime=30s";
    myArgs[9] = "algo=type";

    StressPerfTest test = new StressPerfTest(myArgs, getAxisAliasesMap(),
        getAxisGlobalAliasesMap());

    test.deployTests();
    test.launchTests();
  }

  public static Map<String, String> getAxisAliasesMap() {
    Map<String, String> aliasesMap = new TreeMap<String, String>();
    aliasesMap.put("axis.flow", FlowTest.class.getName());
    aliasesMap.put("axis.add", AdditionTest.class.getName());
    aliasesMap.put("axis.market", MarketplaceTest.class.getName());
    aliasesMap.put("axis.loan", LoanApprovalTest.class.getName());
    return aliasesMap;
  }

  public static Map<String, String[]> getAxisGlobalAliasesMap() {
    Map<String, String[]> globalAliasesMap = new HashMap<String, String[]>();
    globalAliasesMap.put("axis.all", getAxisAliasesMap().keySet().toArray(
        new String[] {}));
    return globalAliasesMap;
  }
}
