/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.axis.test.perf;

import java.util.HashMap;

import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPMessage;

import junit.framework.AssertionFailedError;

import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.util.SOAPUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

/**
 * @author Charles Souillard
 */
public class FlowTest extends AxisPerfTestCase {

  protected String getProcessName() {
    return "flow";
  }
  protected String getProcessNamespace() {
    return "http://example.com/flow";
  }

  public void deploy() {
    deploy(org.ow2.orchestra.test.perf.flow.FlowTest.class
        .getResource(getProcessName() + ".bpel"),
        org.ow2.orchestra.test.perf.flow.FlowTest.class.getResource(getProcessName()
            + ".wsdl"));
  }

  /**
   * Launches an instance of the process.
   *
   * @return time in milliseconds at which the test ended
   */
  public long launch() {
    long endTime = -1;

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("in", BpelXmlUtil.createElementWithContent("Dupont"));
    Message message = new Message(parts);
    SOAPMessage request = SOAPUtil.buildRpcSOAPMessage(
        "http://example.com/flow/submit", message, "submit");

    String endpoint = getDefaultEndpoint("flowPort");
    try {
      SOAPMessage response = SOAPUtil.call(request, endpoint);
      endTime = System.currentTimeMillis();
      if (response != null) {
        SOAPBodyElement result = getSOAPBodyElement(response);
        checkReturn(result, "flowResultABCDE_AE_BF");
      } else {
        assertTrue("No return available", false);
      }
    } catch (Exception e) {
      throw new OrchestraException("Exception while performing test", e);
    }
    return endTime;
  }

  private void checkReturn(SOAPBodyElement result, String name) {
    try {
      assertNotNull(result);
      assertEquals("submit", result.getElementName().getLocalName());
      assertNotNull(result.getFirstChild());
      assertTrue(result.getFirstChild() instanceof Element);
      Element st = (Element) result.getFirstChild();
      assertEquals("out", st.getLocalName());
      assertNotNull(st.getFirstChild());
      assertTrue(st.getFirstChild() instanceof Text);
      Text text = (Text) st.getFirstChild();
      assertEquals(name, text.getData());
    } catch (AssertionFailedError e) {
      System.err.println("result = " + BpelUtil.getNodeAsString(result));
      throw e;
    }
  }

  public void testFlow() {
    deploy();
    launch();
    undeploy();
  }

  public String getAlias() {
    return "axis.flow";
  }
}
