/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.axis.test.receiveFlow;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPMessage;

import junit.framework.AssertionFailedError;

import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.axis.AxisConfiguration;
import org.ow2.orchestra.axis.test.AxisTestCase;
import org.ow2.orchestra.util.SOAPUtil;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class ReceiveFlowTest extends AxisTestCase {

  static class ReceiveFlowThread implements Runnable {

    private Throwable uncaughtException;

    private String id;

    public Throwable getTrowable() {
      return uncaughtException;
    }

    public ReceiveFlowThread(String text) {
      this.id = text;
    }

    public void run() {
      try {
        String part = "<request>" + id + "</request>";
        SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(
            "http://orchestra.ow2.org/tests/receiveFlow/receive", part);
        AxisConfiguration config = AxisConfiguration.getInstance();
        String endpoint = "http://" + config.getHost() + ":" + config.getPort()
            + "/" + config.getWebappName() + "/" + config.getServletPath()
            + "/" + "receiveFlowPort";
        SOAPMessage response = SOAPUtil.call(request, endpoint);
        if (response != null) {
          SOAPBodyElement result = (SOAPBodyElement) response.getSOAPBody()
              .getChildElements().next();
          assertNotNull(result);
          // assertEquals(XmlUtil.toString(result), "result",
          // result.getElementName().getLocalName());
          assertNotNull(result.getFirstChild());
          assertTrue(XmlUtil.toString(result),
              result.getFirstChild() instanceof Text);
          Text text = (Text) result.getFirstChild();
          assertEquals(id, text.getData());
        } else {
          assertTrue("No return available", false);
        }
      } catch (Throwable e) {
        uncaughtException = e;
      }
    }
  }

  /**
   * test.
   */
  public void testAxisReceiveFlow() {
    try {
      String processName = "receiveFlow";
      String processNamespace = "http://orchestra.ow2.org/tests/receiveFlow";
      QName processQName = new QName(processNamespace, processName);

      deploy(processName);

      // Create instance
      String part = "<request>create</request>";
      SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(
          "http://orchestra.ow2.org/tests/receiveFlow/createInstance", part);
      AxisConfiguration config = AxisConfiguration.getInstance();
      String endpoint = "http://" + config.getHost() + ":" + config.getPort()
          + "/" + config.getWebappName() + "/" + config.getServletPath() + "/"
          + "createFlowPort";
      SOAPMessage response = SOAPUtil.call(request, endpoint);
      assertNull(response);

      ReceiveFlowThread firstCall = new ReceiveFlowThread("first");
      ReceiveFlowThread secondCall = new ReceiveFlowThread("second");
      ReceiveFlowThread thirdCall = new ReceiveFlowThread("third");

      Thread t1 = new Thread(firstCall);
      Thread t2 = new Thread(secondCall);
      Thread t3 = new Thread(thirdCall);

      // Do the first call
      t1.start();
      // Do the second call
      t2.start();
      // Do the third call
      t3.start();

      // Wait for answers
      while (t1.isAlive() || t2.isAlive() || t3.isAlive()) {
        t1.join(100);
        t2.join(100);
        t3.join(100);
        if (firstCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread").initCause(firstCall
              .getTrowable());
        }
        if (secondCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread")
              .initCause(secondCall.getTrowable());
        }
        if (thirdCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread").initCause(thirdCall
              .getTrowable());
        }
      }

      // undeploy services
      undeploy(processQName);

    } catch (Throwable e) {
      if (e instanceof AssertionFailedError) {
        throw (AssertionFailedError) e;
      }
      e.printStackTrace();
      assertTrue(false);
    }
  }

}
