/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.axis.test.receiveSeq;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPMessage;

import junit.framework.AssertionFailedError;

import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.axis.AxisConfiguration;
import org.ow2.orchestra.axis.test.AxisTestCase;
import org.ow2.orchestra.util.SOAPUtil;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class ReceiveSeqTest extends AxisTestCase {

  private static final String SOAPACTION1 = "http://orchestra.ow2.org/tests/receiveSeq/receive";

  private static final String SOAPACTION2 = "http://orchestra.ow2.org/tests/receiveSeq/receive2";

  static class ReceiveSeqThread implements Runnable {

    private Throwable uncaughtException;

    private String id;

    private String soapAction;

    private String operation;

    public Throwable getTrowable() {
      return uncaughtException;
    }

    public ReceiveSeqThread(String text, String soapAction, String operation) {
      this.id = text;
      this.soapAction = soapAction;
      this.operation = operation;
    }

    public void run() {
      try {
        String part = "<" + operation + ">" + id + "</" + operation + ">";
        SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(soapAction,
            part);
        AxisConfiguration config = AxisConfiguration.getInstance();
        String endpoint = "http://" + config.getHost() + ":" + config.getPort()
            + "/" + config.getWebappName() + "/" + config.getServletPath()
            + "/" + "receiveSeqPort";
        SOAPMessage response = SOAPUtil.call(request, endpoint);
        if (response != null) {
          SOAPBodyElement result = (SOAPBodyElement) response.getSOAPBody()
              .getChildElements().next();
          assertNotNull(result);
          // assertEquals(XmlUtil.toString(result), "result",
          // result.getElementName().getLocalName());
          assertNotNull(result.getFirstChild());
          assertTrue(XmlUtil.toString(result),
              result.getFirstChild() instanceof Text);
          Text text = (Text) result.getFirstChild();
          assertEquals(id, text.getData());
        } else {
          assertTrue("No return available", false);
        }
      } catch (Throwable e) {
        uncaughtException = e;
      }
    }
  }

  /**
   * test.
   */
  public void testAxisReceiveSeq() {
    try {
      String processName = "receiveSeq";
      String processNamespace = "http://orchestra.ow2.org/tests/receiveSeq";
      QName processQName = new QName(processNamespace, processName);

      deploy(processName);

      // Create instance
      String part = "<request>create</request>";
      SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(
          "http://orchestra.ow2.org/tests/receiveSeq/createInstance", part);
      AxisConfiguration config = AxisConfiguration.getInstance();
      String endpoint = "http://" + config.getHost() + ":" + config.getPort()
          + "/" + config.getWebappName() + "/" + config.getServletPath() + "/"
          + "createSeqPort";
      SOAPMessage response = SOAPUtil.call(request, endpoint);
      assertNull(response);

      ReceiveSeqThread firstCall = new ReceiveSeqThread("first", SOAPACTION1,
          "receive");
      ReceiveSeqThread secondCall = new ReceiveSeqThread("second", SOAPACTION1,
          "receive");
      ReceiveSeqThread thirdCall = new ReceiveSeqThread("third", SOAPACTION1,
          "receive");
      ReceiveSeqThread fourthCall = new ReceiveSeqThread("fourth", SOAPACTION2,
          "receive2");

      Thread t1 = new Thread(firstCall);
      Thread t2 = new Thread(secondCall);
      Thread t3 = new Thread(thirdCall);
      Thread t4 = new Thread(fourthCall);

      // Do the first call
      t1.start();
      // Do the second call
      t2.start();
      // Do the third call
      t3.start();
      // Do the fourth call
      t4.start();

      // Wait for answers
      while (t1.isAlive() || t2.isAlive() || t3.isAlive() || t4.isAlive()) {
        t1.join(100);
        t2.join(100);
        t3.join(100);
        t4.join(100);
        if (firstCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread").initCause(firstCall
              .getTrowable());
        }
        if (secondCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread")
              .initCause(secondCall.getTrowable());
        }
        if (thirdCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread").initCause(thirdCall
              .getTrowable());
        }
        if (fourthCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread")
              .initCause(fourthCall.getTrowable());
        }
      }

      // undeploy services
      undeploy(processQName);

    } catch (Throwable e) {
      if (e instanceof AssertionFailedError) {
        throw (AssertionFailedError) e;
      }
      e.printStackTrace();
      assertTrue(false);
    }
  }

  public void testAxisReceiveSeqWrongOrder() {
    try {
      String processName = "receiveSeq";
      String processNamespace = "http://orchestra.ow2.org/tests/receiveSeq";
      QName processQName = new QName(processNamespace, processName);

      deploy(processName);

      // Create instance
      String part = "<request>create</request>";
      SOAPMessage request = SOAPUtil.buildDocumentSOAPMessage(
          "http://orchestra.ow2.org/tests/receiveSeq/createInstance", part);
      AxisConfiguration config = AxisConfiguration.getInstance();
      String endpoint = "http://" + config.getHost() + ":" + config.getPort()
          + "/" + config.getWebappName() + "/" + config.getServletPath() + "/"
          + "createSeqPort";

      ReceiveSeqThread firstCall = new ReceiveSeqThread("first", SOAPACTION1,
          "receive");
      ReceiveSeqThread secondCall = new ReceiveSeqThread("second", SOAPACTION1,
          "receive");
      ReceiveSeqThread thirdCall = new ReceiveSeqThread("third", SOAPACTION1,
          "receive");
      ReceiveSeqThread fourthCall = new ReceiveSeqThread("fourth", SOAPACTION2,
          "receive2");

      Thread t1 = new Thread(firstCall);
      Thread t2 = new Thread(secondCall);
      Thread t3 = new Thread(thirdCall);
      Thread t4 = new Thread(fourthCall);

      // Do the first call
      t1.start();
      // Do the second call
      t2.start();
      // Do the third call
      t3.start();
      // Do the fourth call
      t4.start();

      Thread.sleep(1000);

      SOAPMessage response = SOAPUtil.call(request, endpoint);
      assertNull(response);

      // Wait for answers
      while (t1.isAlive() || t2.isAlive() || t3.isAlive() || t4.isAlive()) {
        t1.join(100);
        t2.join(100);
        t3.join(100);
        t4.join(100);
        if (firstCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread").initCause(firstCall
              .getTrowable());
        }
        if (secondCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread")
              .initCause(secondCall.getTrowable());
        }
        if (thirdCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread").initCause(thirdCall
              .getTrowable());
        }
        if (fourthCall.getTrowable() != null) {
          throw new AssertionFailedError("Error in thread")
              .initCause(fourthCall.getTrowable());
        }
      }

      // undeploy services
      undeploy(processQName);

    } catch (Throwable e) {
      if (e instanceof AssertionFailedError) {
        throw (AssertionFailedError) e;
      }
      e.printStackTrace();
      assertTrue(false);
    }
  }
}
