package org.ow2.orchestra.jmxclient;

import java.io.IOException;

import javax.naming.ServiceUnavailableException;
import javax.xml.namespace.QName;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.ow2.novabpm.util.Misc;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.jmx.RemoteDeployerMBean;

public class UndeployAction implements ClientAction {

  // create Options object
  private Options options = ClientOptions.createUndeployOptions();

  /**
   * Undeploys a BPEL process
   *
   * @param processQName
   *            name of the process to undeploy
   * @param jmxServiceUrl
   *            url of the jmx service
   * @param jmxObjectName
   *            name of the remote deployer
   * @return <code>true</code> if the process was successfully removed.
   */
  public static boolean undeploy(QName processQName, String jmxServiceUrl,
      String jmxObjectName) {
    boolean result = false;
    try {
      RemoteDeployerMBean remoteDeployer = Misc.getMBeanProxy(RemoteDeployerMBean.class, jmxServiceUrl, jmxObjectName);
      result = remoteDeployer.undeploy(processQName);
    } catch (Exception e) {
      throw new OrchestraException(
          "Exception caught while undeploying process : " + processQName, e);
    }
    return result;
  }

  /**
   * Undeploys a process
   *
   * @param line
   *            parsed command line
   */
  protected static void undeploy(CommandLine line) {
    QName processQName = null;

    String jmxServiceUrl = line.getOptionValue(ClientOptions.JMX_URL_OPTION,
        JMXClient.jmxServiceUrl);
    String jmxObjectName = line.getOptionValue(ClientOptions.JMX_OBJECT_OPTION,
        JMXClient.jmxObjectName);

    if (line.hasOption(ClientOptions.PROCESS_QNAME_OPTION)) {
      processQName = QName.valueOf(line
          .getOptionValue(ClientOptions.PROCESS_QNAME_OPTION));
    }
    boolean result = false;
    try {
      result = undeploy(processQName, jmxServiceUrl, jmxObjectName);
    } catch (Exception e) {
      Throwable cause = e.getCause();
      if ((cause instanceof IOException)
          && (cause.getCause() instanceof ServiceUnavailableException)) {
        System.out
            .println("Error during connection. Please check that Orchestra is started.");
      } else {
        e.printStackTrace();
        System.out.println("Operation failed : "
            + cause.getCause().getMessage());
      }
    }
    if (result) {
      System.out.println("Process " + processQName + " undeployed !");
    } else {
      System.out.println("Problem during undeployment of process "
          + processQName);
    }
  }

  public boolean parseAndExecute(String[] args) {
    // create the parser
    CommandLineParser parser = new GnuParser();
    CommandLine line = null;
    try {
      // parse the command line arguments
      line = parser.parse(options, args);
    } catch (ParseException exp) {
      // oops, something went wrong
      return false;
    }

    undeploy(line);

    return true;
  }

  public Options getOptions() {
    return options;
  }

  public String getDescription() {
    return "To undeploy a BPEL process";
  }
}
