/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import org.ow2.orchestra.runtime.BpelExecution;

/**
 * An automatic activity is one havinf no nested activity.
 * 
 * @author Charles Souillard
 */
public abstract class AutomaticActivity extends AbstractActivity {

  protected AutomaticActivity() {
    super();
  }

  protected AutomaticActivity(String uuid) {
    super(uuid);
  }

  /**
   * Behaviour : execute business code and then sources and proceed.
   * 
   * @param execution
   *            execution.
   */
  protected final void executeActivity(BpelExecution execution) {
    executeBusinessLogic(execution);
    executeSources(execution);
  }

  /**
   * Business logic of this activity without managing pvm execution stuff.
   * 
   * @param execution
   *            execution.
   */
  protected abstract void executeBusinessLogic(BpelExecution execution);
}
