/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.List;
import java.util.Map;

import org.jbpm.pvm.Execution;
import org.jbpm.pvm.Node;
import org.ow2.orchestra.definition.element.Link;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.FlowRuntime;

/**
 * the runtime behaviour of the flow activity.
 *
 * @author Charles Souillard, Tom Baeyens, Goulven Le Jeune
 * @version $Id: Flow.java 1455 2008-04-21 12:57:20Z porcherg $
 */
public class Flow extends AbstractActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  /** list of links of the flow. */
  protected Map<String, Link> links = null;

  protected Flow() {
    super();
  }

  public Flow(String uuid) {
    super(uuid);
  }

  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  public void executeActivity(BpelExecution execution) {
    execution.pushFlowRuntime(new FlowRuntime(links, execution.getNode()
        .getNodes().size()));
    List<Node> nesteNodes = execution.getNode().getNodes();
    if (nesteNodes != null) {
      for (Node child : nesteNodes) {
        Execution childExecution = execution.createExecution();
        childExecution.execute(child);
      }
    }
    //TODO: execution.setState(Execution.STATE_INACTIVE);
  }

  /**
   * Signal method.
   *
   * @param execution
   *            execution
   * @param signal
   *            signal
   */
  @Override
  public void signal(BpelExecution execution, String signal, Map<String, Object> parameters) {
    if (signal == null) {
      // end the execution
      execution.end(Execution.STATE_ENDED, false);

      // this is the child execution returning
      BpelExecution flowExecution = (BpelExecution) execution.getParent();
      if (flowExecution != null && flowExecution.flowChildFinished()) {
        // signal must be non-null to make the distinction with child executions
        // in the first line of this method
        flowExecution.unlock();
        flowExecution.signal("all-finished");
      }
    } else {
      executeSources(execution);
      execution.popFlowRuntime();
    }
  }

  public Map<String, Link> getLinks() {
    return this.links;
  }

  /**
   * @param linkName
   *            linkName
   * @return the link having the given name
   */
  public Link getLink(String linkName) {
    return links.get(linkName);
  }

  public void setLinks(Map<String, Link> links) {
    this.links = links;
  }
}
