/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.Map;

import org.jbpm.pvm.Node;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.lang.evaluator.InvalidExpressionValue;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ForEachRuntime;

/**
 * the runtime behaviour of the forEach activity.
 *
 * @author Tom Baeyens, Charles Souillard
 */
public class ForEach extends AbstractActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  private String counterName;

  private boolean parallel;

  private Expression startCounterValue;

  private Expression finalCounterValue;

  private Expression completionCondition;

  private boolean successfulBranchesOnly;

  private ForEachRuntime forEachRuntime;

  protected ForEach() {
    super();
  }

  public ForEach(String uuid) {
    super(uuid);
  }

  @Override
  public void signal(BpelExecution execution, String signal, Map<String, Object> parameters) {
    executeActivity(execution);
  }

  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  public void executeActivity(BpelExecution execution) {
    if (forEachRuntime == null) {
      int startCounterRuntimeValue;
      try {
        startCounterRuntimeValue = startCounterValue.getEvaluator().evaluateUnsignedInt(execution);
      } catch (Exception e) {
        throw new InvalidExpressionValue("startCounterValue : "
            + startCounterValue, e);
      }
      int finalCounterRuntimeValue;
      try {
        finalCounterRuntimeValue = finalCounterValue.getEvaluator().evaluateUnsignedInt(execution);
      } catch (Exception e) {
        throw new InvalidExpressionValue("finalCounterValue : "
            + finalCounterValue);
      }
      forEachRuntime = new ForEachRuntime(startCounterRuntimeValue,
          finalCounterRuntimeValue);
    }

    if (parallel) {
      parallelFor(execution);
    } else {
      sequentialFor(execution);
    }
  }

  private void sequentialFor(BpelExecution execution) {
    Node node = execution.getNode();
    Node scope = node.getNodes().get(0);
    if (forEachRuntime.loopAgain()) {
      System.out.println(forEachRuntime);
      execution.execute(scope);
    } else {
      executeSources(execution);
    }
  }

  private void parallelFor(BpelExecution execution) {
    throw new OrchestraException("Parallel ForEach is not supported!");
  }

  public String getCounterName() {
    return counterName;
  }

  public void setCounterName(String counterName) {
    this.counterName = counterName;
  }

  public boolean isParallel() {
    return parallel;
  }

  public void setParallel(boolean parallel) {
    this.parallel = parallel;
  }

  public Expression getStartCounterValue() {
    return startCounterValue;
  }

  public void setStartCounterValue(Expression startCounterValue) {
    this.startCounterValue = startCounterValue;
  }

  public Expression getFinalCounterValue() {
    return finalCounterValue;
  }

  public void setFinalCounterValue(Expression finalCounterValue) {
    this.finalCounterValue = finalCounterValue;
  }

  public Expression getCompletionCondition() {
    return completionCondition;
  }

  public void setCompletionCondition(Expression completionCondition) {
    this.completionCondition = completionCondition;
  }

  public boolean isSuccessfulBranchesOnly() {
    return successfulBranchesOnly;
  }

  public void setSuccessfulBranchesOnly(boolean successfulBranchesOnly) {
    this.successfulBranchesOnly = successfulBranchesOnly;
  }
}
