/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.logging.Logger;

import org.jbpm.pvm.Node;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.runtime.BpelExecution;

/**
 * the runtime behaviour of the repeatUntil activity.
 * 
 * @author Tom Baeyens
 */
public class RepeatUntil extends AbstractActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  private static Logger log = Logger.getLogger(RepeatUntil.class.getName());

  protected Expression condition;

  protected RepeatUntil() {
    super();
  }

  public RepeatUntil(String uuid) {
    super(uuid);
  }

  /**
   * business logic.
   * 
   * @param execution
   *            execution
   */
  public void executeActivity(BpelExecution execution) {
    Node node = execution.getNode();
    Node activity = node.getNodes().get(0);
    execution.execute(activity);
  }

  /**
   * Signal method of the NodeBehaviour.
   * 
   * @param execution
   *            execution
   * @param signal
   *            signal
   */
  public void signal(BpelExecution execution, String signal) {
    Node node = execution.getNode();
    Node activity = node.getNodes().get(0);
    boolean conditionResult = condition.getEvaluator().evaluateBoolean(execution);
    if (conditionResult) {
      executeSources(execution);
    } else {
      execution.execute(activity);
    }
  }

  public Expression getCondition() {
    return condition;
  }

  public void setCondition(Expression condition) {
    this.condition = condition;
  }

}
