/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.ow2.orchestra.exception.Fault;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;
import org.ow2.orchestra.var.BpelFaultMessage;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * the runtime behaviour of the rethrow activity.
 *
 * @author Tom Baeyens, Charles Souillard
 */
public class Rethrow extends AbstractActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  private static Logger log = Logger.getLogger(Rethrow.class.getName());

  protected String cathVariableName;
  protected QName catchFaultQName;

  protected Rethrow() {
    super();
  }

  public Rethrow(String uuid) {
    super(uuid);
  }

  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  public void executeActivity(BpelExecution execution) {
    if (this.cathVariableName != null) {
      ScopeRuntime varScope = execution.getScopeRuntimeWithVariable(cathVariableName);
      Object catchValue = varScope.getVariable(cathVariableName, execution).getValue();
      List<BpelFaultMessage> faults = new ArrayList<BpelFaultMessage>();
      if (catchValue instanceof Message) {
        faults.add(new BpelFaultMessage(catchFaultQName,((Message)catchValue).getParts()));
      } else if (catchValue instanceof Element) {
        // TODO
      }
      ScopeRuntime currentScope = execution.getScopeRuntime();
      while (varScope != currentScope) {
        currentScope = execution.popScopeRuntime();
      }
      //we need to be on the father of varScope
      currentScope = execution.popScopeRuntime();

      log.fine("this fault (" + catchFaultQName + ") is rethrowed");
      throw new Fault(faults);
    } else {
      throw new OrchestraException("Error: impossible to find the fault variable to rethrow");
    }
  }


  public String getCathVariableName() {
    return cathVariableName;
  }


  public void setCathVariableName(String cathVariableName) {
    this.cathVariableName = cathVariableName;
  }


  public QName getCathFaultName() {
    return catchFaultQName;
  }


  public void setCathFaultName(QName cathFaultName) {
    this.catchFaultQName = cathFaultName;
  }

}
