/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.List;
import java.util.Map;

import org.jbpm.pvm.Node;
import org.jbpm.pvm.impl.NodeImpl;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.BpelExecution;

/**
 * implementation of the bpel sequence.
 *
 * @author Charles Souillard, Tom Baeyens
 * @version $Id: Sequence.java 1409 2008-04-08 09:39:42Z porcherg $
 */
public class Sequence extends AbstractActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  protected Sequence() {
    super();
  }

  public Sequence(String uuid) {
    super(uuid);
  }

  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  public void executeActivity(BpelExecution execution) {
    // get the current sequence node
    Node node = execution.getNode();
    // check that the node's previousNeeded is activated
    if (!node.isPreviousNeeded()) {
      throw new OrchestraException("behaviour " + Scope.class.getName()
          + " requires that node.isPreviousNeeded() is set to true");
    }
    // get the first nested nodes in the sequence
    Node firstNode = node.getNodes().get(0);
    // record that we're going to execute the first node in the sequence
    // record("next-index[" + execution.getNode().getName() + ",0]");
    // execute the first node
    execution.execute(firstNode);
  }

  /**
   * Signal method of the NodeBehaviour.
   *
   * @param execution
   *            execution
   * @param signal
   *            signal
   */
  @Override
  public void signal(BpelExecution execution, String signal, Map<String, Object> parameters) {
    NodeImpl node = execution.getNode();
    List<Node> nestedNodes = node.getNodes();

    Node previousNode = (NodeImpl) execution.getPreviousNode();

    int nextNodeIndex = nestedNodes.indexOf(previousNode) + 1;

    // if the index is equal to the size of the list of nested nodes
    if (nestedNodes.size() == nextNodeIndex) {
      // record("done[" + execution.getNode().getName() + "]");
      executeSources(execution);
    } else {
      // else just increment the index and execute the next node
      Node nextNode = nestedNodes.get(nextNodeIndex);
      // record the next index
      // record("next-index[" + execution.getNode().getName() + "," +
      // nextNodeIndex + "]");
      // execute the next node
      execution.execute(nextNode);
    }
  }

}
