/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.ow2.orchestra.exception.BpelFaultException;
import org.ow2.orchestra.exception.Fault;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.util.XmlConstants;
import org.ow2.orchestra.var.BpelFaultMessage;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * the runtime behaviour of the throw activity.
 *
 * @author Tom Baeyens, Charles Souillard
 */
public class Throw extends AbstractActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  private static Logger log = Logger.getLogger(Throw.class.getName());


  /** faultQName attribute of this throw element. */
  private QName faultQName;
  /** faultVariable attribute of this throw element. */
  private String faultVariable;



  protected Throw() {
    super();
  }

  public Throw(String uuid) {
    super(uuid);
  }

  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  public void executeActivity(BpelExecution execution) {
    if (isBpelFault()) {
      Iterator<BpelFaultException> faultIt = BpelFaultException.getBpelFaults().iterator();
      BpelFaultException res = null;
      while (faultIt.hasNext() && res == null) {
        BpelFaultException current = faultIt.next();
        if (this.getFaultName().getLocalPart().toLowerCase().equals(current.getClass().getSimpleName().toLowerCase())) {
          res = current;
        }
      }
      if (res != null) {
        log.fine("Bpel Fault throwed: " + res.getClass().getName());
        throw res;
      } else {
        throw new OrchestraException("No Bpel Fault correspond to this fault: " + this.getFaultName());
      }
    } else {
      Fault userFault = getUserFaultVariable(execution);
      log.fine("User Fault throwed: " + userFault);
      throw userFault;
    }
  }

  private Fault getUserFaultVariable(BpelExecution execution) {
    Fault userFault = null;
    if (this.getFaultVariable() != null) {
      // get the fault variable
      ScopeRuntime scopeRuntime = execution.getScopeRuntime();
      VariableRuntime faultV = scopeRuntime.getVariable(this.getFaultVariable(), execution);
      List<BpelFaultMessage> faults = new ArrayList<BpelFaultMessage>();
      Map<String, Element> parts = null;
      Object val = faultV.getValue();

      if (val instanceof Element) {
        parts = new HashMap<String, Element>();
        parts.put(((Element)val).getNodeName(), (Element)val);
      } else if (val instanceof Message) {
        parts = ((Message)val).getParts();
      } else {
        throw new OrchestraException(
          "the type of the value of the variable is unknown");
      }
      // Creation of the user fault
      faults.add(new BpelFaultMessage(this.getFaultName(),parts));
      userFault = new Fault(faults);
    }
    return userFault;
  }

  private boolean isBpelFault() {
    boolean res = false;
    if (this.getFaultName().getNamespaceURI().equals(XmlConstants.XMLNS_BPEL_2_0_EXECUTABLE)) {
      res = true;
    }
    return res;
  }

  public QName getFaultName() {
    return faultQName;
  }

  public void setFaultName(QName faultName) {
    this.faultQName = faultName;
  }

  public String getFaultVariable() {
    return faultVariable;
  }

  public void setFaultVariable(String faultVariable) {
    this.faultVariable = faultVariable;
  }

}
