/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.pvm.Node;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.runtime.BpelExecution;

/**
 * the runtime behaviour of the while activity.
 *
 * @author Charles Souillard
 */
public class While extends AbstractActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  private static Logger log = Logger.getLogger(While.class.getName());

  protected Expression condition;

  protected While() {
    super();
  }

  public While(String uuid) {
    super(uuid);
  }

  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  public void executeActivity(BpelExecution execution) {
    executeLoop(execution);
  }

  /**
   * Signal method of the NodeBehaviour.
   *
   * @param execution
   *            execution
   * @param signal
   *            signal
   */
  @Override
  public void signal(BpelExecution execution, String signal, Map<String, Object> parameters) {
    executeLoop(execution);
  }

  private void executeLoop(BpelExecution execution) {
    Node node = execution.getNode();
    Node firstNode = node.getNodes().get(0);

    boolean whileResult = condition.getEvaluator().evaluateBoolean(execution);
    if (whileResult) {
      if (log.isLoggable(Level.FINE)) {
        log.fine("while condition : " + condition + " evaluated to true");
      }
      execution.execute(firstNode);
    } else {
      executeSources(execution);
    }
  }

  public Expression getCondition() {
    return condition;
  }

  public void setCondition(Expression condition) {
    this.condition = condition;
  }

}
