/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity.extensions;

import java.util.List;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.definition.activity.AutomaticActivity;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.PartnerLinkRuntime;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.services.itf.TestValidator;

public class Test extends AutomaticActivity {

  protected List<String> varNames;

  protected List<String> plnkNames;

  protected String event;

  protected Test() {
    super();
  }

  public Test(String uuid) {
    super(uuid);
  }

  protected void executeBusinessLogic(BpelExecution execution) {
    TestValidator testValidator = (TestValidator) Environment.getCurrent().get(TestValidator.DEFAULT_KEY);
    QName processQName = execution.getProcessDefinition().getQName();
    BpelExecution instance = execution.getProcessInstance();
    if (varNames != null) {
      for (String variableName : varNames) {
        VariableRuntime variableRuntime = execution
            .getVariableByName(variableName);
        testValidator.recordVariable(instance, processQName, variableRuntime);
      }
    }
    if (plnkNames != null) {
      for (String partnerLinkName : plnkNames) {
        PartnerLinkRuntime partnerLinkRuntime = execution
            .getPartnerLinkRuntime(partnerLinkName);
        testValidator.recordPartnerLink(instance, processQName,
            partnerLinkRuntime);
      }
    }
    if (event != null) {
      testValidator.recordEvent(instance, processQName, event);
    }
  }

  public void setVarNames(List<String> varNames) {
    this.varNames = varNames;
  }

  public void setPlnkNames(List<String> plnkNames) {
    this.plnkNames = plnkNames;
  }

  public void setEvent(String event) {
    this.event = event;
  }
}
