/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import java.util.List;
import java.util.Map.Entry;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.jbpm.pvm.impl.NodeImpl;
import org.ow2.orchestra.exception.Fault;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.var.BpelFaultMessage;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 *
 * @author Nicolas SALATGE - eBM WebSourcing
 */
public class Catch {
  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;
  private static Logger log = Logger.getLogger(Catch.class.getName());

  /** faultQName attribute of this catch element. */
  private QName faultQName;
  /** faultVariable attribute of this catch element. */
  private Variable faultVariable;
  /** faultMessageType attribute of this catch element. */
  private QName faultMessageType;
  /** faultElement attribute of this catch element. */
  private QName faultElement;

  private NodeImpl activity;

  public Catch() {
    super();
  }

  public QName getFaultName() {
    return faultQName;
  }
  public void setFaultName(QName faultName) {
    this.faultQName = faultName;
  }
  public Variable getFaultVariable() {
    return faultVariable;
  }
  public void setFaultVariable(Variable faultVariable) {
    this.faultVariable = faultVariable;
  }
  public QName getFaultMessageType() {
    return faultMessageType;
  }
  public void setFaultMessageType(QName faultMessageType) {
    this.faultMessageType = faultMessageType;
  }
  public QName getFaultElement() {
    return faultElement;
  }
  public void setFaultElement(QName faultElement) {
    this.faultElement = faultElement;
  }

  public Throwable getFirstCause(Throwable e) {
    Throwable first = e;
    Throwable index = e.getCause();
    while (index != null) {
      first = index;
      index = index.getCause();
    }
    return first;
  }

  /**
   * Match an exception with this catch
   * @param e
   * @return the fault message or null
   */
  public Message match(BpelExecution execution, Exception e) {
    Message res = null;

    Throwable ex = this.getFirstCause(e);
    if (ex instanceof Fault) {
      Fault f = (Fault)ex;
      List<BpelFaultMessage> faults = f.getFaults();
      java.util.Iterator<BpelFaultMessage> faultsIt = faults.iterator();
      BpelFaultMessage currentFault = null;
      while (faultsIt.hasNext() && res == null) {
        currentFault = faultsIt.next();

        if (this.getFaultName() == null && this.getFaultMessageType() != null) {
          log.fine("Match the messageType: " + this.getFaultMessageType());
          javax.wsdl.Message msg = execution.getMessage(this.getFaultMessageType());


          Message msgRt = new Message(msg);
          Message curr = new Message(currentFault.getParts());

          if (this.compareMessage(curr, msgRt)) {
            res = currentFault;
            log.fine("MessageType: " + this.getFaultMessageType() + " matched");
          }
        } else if (this.getFaultName() == null && this.getFaultElement() != null) {
          log.fine("Match the element: " + this.getFaultElement());
          if (currentFault.getParts() != null) {
            if (currentFault.getParts().size() > 1) {
              throw new OrchestraException("this message is not an element. It has several parts");
            }
            Element elmt = currentFault.getParts().values().iterator().next();

            if (this.getFaultElement().getLocalPart().equals(elmt.getLocalName())
                && this.getFaultElement().getNamespaceURI().equals(elmt.getNamespaceURI())) {
              res = currentFault;
              log.fine("Element: " + this.getFaultElement() + " matched");
            }
          }
        } else if (currentFault.getFaultName().equals(this.getFaultName())) {
          log.fine("Match the fault name: " + this.getFaultName());
          res = currentFault;
        }
      }
    }

    if (this.faultQName == null
        && this.faultVariable == null
        && this.faultMessageType == null
        && this.faultElement == null) {
      res = new Message();
    }
    return res;
  }

  public void setActivity(NodeImpl activity) {
    this.activity = activity;
  }

  public boolean compareMessage(Message msg1, Message msg2) {
    for (Entry<String, Element> entry : msg1.getParts().entrySet()) {
      if (!msg2.getParts().containsKey(entry.getKey())
          && !msg2.getParts().get(entry.getKey()).getLocalName().equals(entry.getValue().getLocalName())
          && !msg2.getParts().get(entry.getKey()).getNamespaceURI().equals(entry.getValue().getNamespaceURI())) {
        return false;

      }
    }
    return true;
  }


  public String toString() {
    String result = "<catch";
    if (this.faultQName != null) {
      result += " faultQName=\"" + faultQName + "\"";
    }
    if (this.faultVariable != null) {
      result += " faultVariable=\"" + faultVariable + "\"";
    }
    if (this.faultMessageType != null) {
      result += " faultMessageType=\"" + faultMessageType + "\"";
    }
    if (this.faultElement != null) {
      result += " faultElement=\"" + faultElement + "\"";
    }
    result += ">";
    result += this.activity;
    result += "</catch>\n";
    return result;
  }

  public NodeImpl getActivity() {
    return activity;
  }
}
