/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.wsdl.Message;
import javax.wsdl.Part;
import javax.xml.namespace.QName;

import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.wsdl.Property;
import org.ow2.orchestra.wsdl.PropertyAlias;

/**
 * @author Charles Souillard
 */
public class CorrelationSet implements Serializable {
  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;
  /** name attribute of this CorrelationSet. */
  private String name;
  /** list of properties of this correlationSet. */
  private Map<QName, Property> properties;

  public List<PropertyAlias> getPropertyAliasRefs(BpelExecution execution, final QName messageType) {
    List<PropertyAlias> propertyRefs = new ArrayList<PropertyAlias>();
    for (QName newPropertyName : properties.keySet()) {
      PropertyAlias newPropertyAlias = execution.getPropertyAliasWithMessageType(newPropertyName, messageType);
      if (newPropertyAlias == null) {
        Message message = execution.getWsdlInfos().getMessage(messageType);
        if (message.getParts().size() == 1) {
          QName elementQName = ((Part)message.getParts().values().iterator().next()).getElementName();
          if (elementQName != null) {
            newPropertyAlias = execution.getPropertyAliasWithElement(newPropertyName, elementQName);
          }
        }
      }
      if (newPropertyAlias == null) {
        // TODO: this exception should never be thrown, 
        // matching propertyAlias is checked during static analysis
        // see bpel specification p 78
        throw new OrchestraException("No property alias found for property " 
            + newPropertyName + " in message type " + messageType);
      } else {
        propertyRefs.add(newPropertyAlias);
      }
    }
    return propertyRefs;
  }

  public String getName() {
    return this.name;
  }
  public void setName(String name) {
    this.name = name;
  }
  public Map<QName, Property> getProperties() {
    return this.properties;
  }
  public void setProperties(Map<QName, Property> properties) {
    this.properties = properties;
  }

}
