/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import javax.xml.namespace.QName;

import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.lang.PropertyQuery;
import org.ow2.orchestra.lang.VariableQuery;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Goulven Le Jeune, Charles Souillard, Candy Chlad, Stanislas Giraudet
 *         De Boudemange
 */
public class FromTo {

  public enum Variant {
    VAR, PLNK, PROP, EXPR, LITERAL
  }

  protected Variant variant;

  /** expressionLanguage common attribute of this from/to. */
  protected String expressionLanguage;

  /** variable common attribute of this from/to. */
  protected String variable;

  /** part common attribute of this from/to. */
  protected String part;

  /** property common attribute of this from/to. */
  protected QName property;

  /** partnerLink common attribute of this from/to. */
  protected String partnerLink;

  /** endpointReference common attribute of this from/to. */
  protected String endpointReference;

  /** query common attribute of this from/to. */
  protected VariableQuery variableQuery;

  protected PropertyQuery propertyQuery;

  /** expression. */
  protected Expression expression;

  protected FromTo() {
    super();
  }

  public Element getContextNode(BpelExecution execution) {
    if (variable == null) {
      return null;
    }
    VariableRuntime variableRuntime = execution
        .getVariableByName(this.variable);
    Variable variableDefinition = variableRuntime.getVariableDefinition();
    if (variableDefinition.getMessageType() != null) {
      return ((Message) variableRuntime.getValue()).getPartValue(part);
    } else {
      return (Element) variableRuntime.getValue();
    }
    /*
     * if (variable.getVariableDefinition().getMessageType() != null && part !=
     * null) { // if this part is using a type and if the type is a schema type,
     * we need to convert it javax.wsdl.Message message =
     * execution.getProcess().getWsdlInfos().getMessage(variable.getVariableDefinition().getMessageType());
     * 
     * QName partTypeQName = message.getPart(part).getTypeName(); Message
     * varValue = (Message)variable.getValue(); Element rootElement =
     * varValue.getPartValue(part); if (partTypeQName != null &&
     * BpelUtil.isXmlSchemaNs(partTypeQName.getNamespaceURI())) { return
     * getElementContentAsString(partTypeQName, rootElement); } else { return
     * rootElement; } } else if
     * (variable.getVariableDefinition().getMessageType() != null) { return
     * ((Message)variable.getValue()).getPartValue(part); } else if
     * (variable.getVariableDefinition().getElement() != null) { return
     * (Element) variable.getValue(); } else if
     * (variable.getVariableDefinition().getType() != null) { return (Element)
     * variable.getValue(); QName type =
     * variable.getVariableDefinition().getType(); if
     * (BpelUtil.isXmlSchemaNs(type.getNamespaceURI())) { return
     * getElementContentAsString(type, (Element)variable.getValue()); } else {
     * return (Element) variable.getValue(); } } return null;
     */
  }

  public String getExpressionLanguage() {
    return expressionLanguage;
  }

  public String getPart() {
    return part;
  }

  public String getPartnerLink() {
    return partnerLink;
  }

  public QName getProperty() {
    return property;
  }

  public String getVariable() {
    return variable;
  }

  public String getEndpointReference() {
    return endpointReference;
  }

  public Variant getVariant() {
    return this.variant;
  }

  public void setVariant(Variant variant) {
    this.variant = variant;
  }

  public void setEndpointReference(String endpointReference) {
    this.endpointReference = endpointReference;
  }

  public void setExpressionLanguage(String expressionLanguage) {
    this.expressionLanguage = expressionLanguage;
  }

  public void setPart(String part) {
    this.part = part;
  }

  public void setPartnerLink(String partnerLink) {
    this.partnerLink = partnerLink;
  }

  public void setProperty(QName property) {
    this.property = property;
  }

  public void setVariable(String variable) {
    this.variable = variable;
  }

  public Expression getExpression() {
    return expression;
  }

  public void setExpression(Expression expression) {
    this.expression = expression;
  }

  public VariableQuery getVariableQuery() {
    return variableQuery;
  }

  public void setVariableQuery(VariableQuery variableQuery) {
    this.variableQuery = variableQuery;
  }

  public PropertyQuery getPropertyQuery() {
    return propertyQuery;
  }

  public void setPropertyQuery(PropertyQuery propertyQuery) {
    this.propertyQuery = propertyQuery;
  }
}
