package org.ow2.orchestra.definition.element;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.datatype.Duration;

import org.jbpm.env.Environment;
import org.jbpm.env.session.Timer;
import org.jbpm.env.session.TimerSession;
import org.jbpm.pvm.Activity;
import org.jbpm.pvm.Execution;
import org.jbpm.pvm.impl.NodeImpl;
import org.jbpm.pvm.impl.ObjectReference;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.MutexRepository;


public class OnAlarm {

  private static Logger log = Logger.getLogger(OnAlarm.class.getName());
 
  private Expression durationExpression;

  private Expression deadlineExpression;
  
  protected NodeImpl activity;

  public void setDurationExpression(Expression durationExpression) {
    this.durationExpression = durationExpression;
  }
  public void setDeadlineExpression(Expression deadlineExpression) {
    this.deadlineExpression = deadlineExpression;
  }
  public NodeImpl getActivity() {
    return activity;
  }
  public void setActivity(NodeImpl activity) {
    this.activity = activity;
  }
  public Expression getDurationExpression() {
    return durationExpression;
  }
  public Expression getDeadlineExpression() {
    return deadlineExpression;
  }
  
  
  public static class SignalOnAlarmActivity implements Activity {
    protected NodeImpl onAlarmNode;

    protected SignalOnAlarmActivity() { }
    
    public SignalOnAlarmActivity(NodeImpl onAlarmNode) {
      this.onAlarmNode = onAlarmNode;
    }
    
    public void execute(Execution execution) throws Exception {
      BpelExecution bpelInstance = (BpelExecution) execution.getProcessInstance();
      Map<String, Object> parameters = new HashMap<String, Object>();
      parameters.put("onAlarm", onAlarmNode);
      synchronized (MutexRepository.getInstanceMutex(bpelInstance)) {
        execution.signal(parameters);
      }
    }
  }
  
  public static Timer createTimer(BpelExecution execution, Expression deadlineExpression, 
      Expression durationExpression, ObjectReference<Activity> activityReference) {
    Date date = null;

    if (deadlineExpression != null) {
      date = deadlineExpression.getEvaluator().evaluateDeadline(execution).toGregorianCalendar().getTime();
      if (log.isLoggable(Level.FINE)) {
        log.fine("waiting until " + date);
      }

    } else if (durationExpression != null) {
      Duration duration = durationExpression.getEvaluator().evaluateDuration(execution);
      date = new Date();
      duration.addTo(date);
      if (log.isLoggable(Level.FINE)) {
        log.fine("waiting for " + duration);
      }

    } else {
      throw new OrchestraException("invalid timer definition");
    }
    
    Timer timer = new Timer(execution);
    timer.setActivityReference(activityReference);
    timer.setDueDate(date);
    Environment.getCurrent().get(TimerSession.class).schedule(timer);
    return timer;
  }
}
