/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.jbpm.pvm.impl.NodeImpl;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.CorrelationSetRuntime;
import org.ow2.orchestra.runtime.ScopeRuntime;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.services.MutexRepository;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.ReceivingElement;
import org.ow2.orchestra.var.Message;

/**
 * @author Charles Souillard, Guillaume Porcher
 */
public class OnMessage implements ReceivingElement {

  protected long dbid;
  protected List<Correlation> correlations;
  protected String messageExchange = "";
  protected String operation;
  protected String partnerLinkName;
  /** fromParts management. */
  protected List<Copy> inCopies;
  protected String variable = null;
  protected QName portTypeQName = null;
  protected Variable anonymousInboundMessageVariable;

  protected NodeImpl activity;

  private static Logger log = Logger.getLogger(OnMessage.class.getName());


  public long getDbid() {
    return dbid;
  }
  public QName getPortTypeQName() {
    return portTypeQName;
  }
  public void setPortTypeQName(QName portTypeQName) {
    this.portTypeQName = portTypeQName;
  }
  public List<Copy> getInCopies() {
    return inCopies;
  }
  public void setInCopies(List<Copy> inCopies) {
    this.inCopies = inCopies;
  }
  public List<Correlation> getCorrelations() {
    return correlations;
  }
  public void setCorrelations(List<Correlation> correlations) {
    this.correlations = correlations;
  }
  public String getMessageExchange() {
    return messageExchange;
  }
  public void setMessageExchange(String messageExchange) {
    this.messageExchange = messageExchange;
  }
  public String getOperation() {
    return operation;
  }
  public void setOperation(String operation) {
    this.operation = operation;
  }
  public String getPartnerLinkName() {
    return partnerLinkName;
  }
  public void setPartnerLinkName(String partnerLinkName) {
    this.partnerLinkName = partnerLinkName;
  }


  public Variable getAnonymousInboundMessageVariable() {
    return anonymousInboundMessageVariable;
  }

  public void setAnonymousInboundMessageVariable(
      Variable anonymousInboundMessageVariable) {
    this.anonymousInboundMessageVariable = anonymousInboundMessageVariable;
  }

  public String getVariable() {
    return variable;
  }

  public void setVariable(String variable) {
    this.variable = variable;
  }

  public NodeImpl getActivity() {
    return activity;
  }
  public void setActivity(NodeImpl activity) {
    this.activity = activity;
  }
  public OperationKey getOperationKey(QName processQName) {
    return new OperationKey(processQName, portTypeQName, operation);
  }

  public void receive(Message message, BpelExecution execution) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("Receiving message : " + message);
    }
    // initiate correlationSets
    initiateCs(execution, message);
    if (variable != null) {
      // we store the variable
      VariableRuntime variableRuntime = execution.getVariableByName(variable);
      variableRuntime.setValue(message);
    } else if (inCopies != null && !inCopies.isEmpty()) {
      ScopeRuntime scopeRuntime = execution.getScopeRuntime();
      VariableRuntime anonymousOutboundMessageVariableRuntime = scopeRuntime
          .createVariable(execution, this.anonymousInboundMessageVariable);

      anonymousOutboundMessageVariableRuntime.setValue(message);
      for (Copy copy : this.inCopies) {
        copy.execute(execution);
      }
      scopeRuntime.removeVariable(execution,
          this.anonymousInboundMessageVariable);
    }
  }

  private void initiateCs(BpelExecution bpelExecution, Message incomingMessage) {
    if (correlations != null) {
      synchronized (MutexRepository.getInstanceMutex(bpelExecution)) {
        // we calcul the values for each correlationSet
        for (Correlation correlation : correlations) {
          CorrelationSetRuntime correlationSetRuntime =
            bpelExecution.getCorrelationSetByName(correlation.getCorrelationSet().getName());
          QName messageType = bpelExecution.getPortType(
              this.getPortTypeQName()).getOperation(this.getOperation(),null, null)
              .getInput().getMessage().getQName();
          correlationSetRuntime.checkCSValues(bpelExecution, incomingMessage, messageType,
              correlation.getInitiate());
        }
      }
    }
  }

}
