/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.PartnerLinkRuntime;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Goulven Le Jeune, Charles Souillard, Candy Chlad, Stanislas Giraudet
 *         De Boudemange
 */
public class To extends FromTo {
  private static Logger log = Logger.getLogger(To.class.getName());

  public void setValue(BpelExecution execution, Object value, boolean keepSrcElementName) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("Setting to value of " + toString());
    }
    if (value instanceof Message) {
      String toMessageVariableName = null;
      if (variant == Variant.VAR) {
        toMessageVariableName = this.variable;

      } else if (variant == Variant.EXPR) {
        throw new OrchestraException("Unsupported from message to Expression");
      }
      VariableRuntime var = execution.getVariableByName(toMessageVariableName);
      Message fromMessage = (Message) value;
      var.setValue(fromMessage.duplicate());
      return;
    }

    if (variant == Variant.EXPR) {
      expression.getEvaluator().assign(execution, value);
    } else if (variant == Variant.VAR) {
      VariableRuntime var = execution.getVariableByName(variable);
      if (part == null) {
        if (variableQuery == null) {
          var.setValue(value, keepSrcElementName);
        } else {
          Element contextNode = (Element) var.getValue();
          variableQuery.getEvaluator().assign(contextNode, execution, value);
        }
      } else {
        if (variableQuery == null) {
          Message mess = (Message) var.getValue();
          mess.setPart(part, value);
        } else {
          Element contextNode = ((Message) var.getValue()).getPartValue(part);
          variableQuery.getEvaluator().assign(contextNode, execution, value);
        }
      }
    } else if (variant == Variant.PROP) {
      Element contextNode = getContextNode(execution);
      propertyQuery.getEvaluator().assign(contextNode, value, execution);
    } else if (variant == Variant.PLNK) {
      PartnerLinkRuntime partnerLinkRuntime = execution
          .getPartnerLinkRuntime(partnerLink);
      if (endpointReference != null) {
        if (endpointReference.equals("myRole")) {
          partnerLinkRuntime.setMyRoleEndPointReference((Element) value);
        } else {
          partnerLinkRuntime.setPartnerRoleEndPointReference((Element) value);
        }
      } else {
        partnerLinkRuntime.setPartnerRoleEndPointReference((Element) value);
      }
    } else {
      throw new OrchestraException("Unknow VARIANT TYPE : " + variant);
    }
  }

  /**
   * toString method.
   *
   * @return String string
   */
  public String toString() {
    String result = "<to";
    if (expressionLanguage != null) {
      result += "  expressionLanguage=\"" + this.expressionLanguage + "\"";
    }
    if (variable != null) {
      result += "  variable=\"" + this.variable + "\"";
    }
    if (part != null) {
      result += "  part=\"" + this.part + "\"";
    }
    if (property != null) {
      result += "  property=\"" + this.property + "\"";
    }
    if (partnerLink != null) {
      result += "  partnerLink=\"" + this.partnerLink + "\"";
    }
    result += ">\n";
    if (variableQuery != null) {
      result += variableQuery.toString();
    }
    result += "</to>\n";
    return result;
  }
}
