/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the JBPM BPEL PUBLIC LICENSE AGREEMENT as
 * published by JBoss Inc.; either version 1.0 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.exception;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.exception.faults.ConflictingReceive;
import org.ow2.orchestra.exception.faults.ConflictingRequest;
import org.ow2.orchestra.exception.faults.InvalidReply;
import org.ow2.orchestra.exception.faults.InvalidVariables;
import org.ow2.orchestra.exception.faults.JoinFailure;
import org.ow2.orchestra.exception.faults.MismatchedAssignmentFailure;
import org.ow2.orchestra.exception.faults.MissingReply;
import org.ow2.orchestra.exception.faults.MissingRequest;
import org.ow2.orchestra.exception.faults.RepeatedCompensation;
import org.ow2.orchestra.exception.faults.ScopeInitializationFailure;
import org.ow2.orchestra.exception.faults.SelectionFailure;
import org.ow2.orchestra.exception.faults.SubLanguageExecutionFault;
import org.ow2.orchestra.exception.faults.UninitializedPartnerRole;
import org.ow2.orchestra.exception.faults.UninitializedVariable;
import org.ow2.orchestra.exception.faults.UnsupportedReference;

/**
 * @author Juan Cantu
 * @version $Revision: 1.2 $ $Date: 2006/10/29 06:13:41 $
 */
public class BpelFaultException extends OrchestraException {

  private Object faultInstance;

  private static final long serialVersionUID = 1L;

  private static List<BpelFaultException> bpelFaults;

  static {
    bpelFaults = new ArrayList<BpelFaultException>();
    bpelFaults.add(new ConflictingReceive());
    bpelFaults.add(new ConflictingRequest());
    bpelFaults.add(new InvalidReply());
    bpelFaults.add(new InvalidVariables());
    bpelFaults.add(new JoinFailure());
    bpelFaults.add(new MismatchedAssignmentFailure());
    bpelFaults.add(new MissingReply());
    bpelFaults.add(new MissingRequest());
    bpelFaults.add(new RepeatedCompensation());
    bpelFaults.add(new ScopeInitializationFailure());
    bpelFaults.add(new SelectionFailure());
    bpelFaults.add(new SubLanguageExecutionFault());
    bpelFaults.add(new UninitializedPartnerRole());
    bpelFaults.add(new UninitializedVariable());
    bpelFaults.add(new UnsupportedReference());
  }
  
  public BpelFaultException() {
    super();
  }

  public BpelFaultException(String message, Exception e) {
    super(message, e);
  }

  public BpelFaultException(Object faultInstance) {
    super(faultInstance.toString());
    this.faultInstance = faultInstance;
  }

  public Object getFaultInstance() {
    return faultInstance;
  }


  public static List<BpelFaultException> getBpelFaults() {
    return bpelFaults;
  }
}
