/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx;

import java.io.InputStream;
import java.lang.management.ManagementFactory;
import java.util.logging.LogManager;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.jbpm.env.Environment;
import org.jbpm.env.EnvironmentFactory;
import org.jbpm.jobexecutor.JobExecutor;
import org.ow2.orchestra.env.BpelEnvironmentParser;
import org.ow2.orchestra.env.EnvFactoryRepository;
import org.ow2.orchestra.exception.OrchestraException;

public class JMXAgent implements ServletContextListener {

  private MBeanServer mbs;

  private ObjectName objectName;

  private JMXConnectorServer cs;

  private JMXConfiguration configuration = JMXConfiguration.getInstance();

  // Default environment for deployed services
  public static String defaultEnvConfigFile = "environment.xml";

  public void contextInitialized(ServletContextEvent arg0) {
    try {
      if (System.getSecurityManager() == null) {
        System.setSecurityManager(new SecurityManager());
      }
      // start rmiregistry
      java.rmi.registry.LocateRegistry.createRegistry(configuration.getPort());

      mbs = ManagementFactory.getPlatformMBeanServer();
      RemoteDeployer remoteDeployer = new RemoteDeployer();
      // Uniquely identify the MBeans and register them with the MBeanServer
      objectName = ObjectName.getInstance(configuration.getObjectName());
      mbs.registerMBean(remoteDeployer, objectName);
      JMXServiceURL url = new JMXServiceURL(configuration.getServiceUrl());
      cs = JMXConnectorServerFactory.newJMXConnectorServer(url, null, mbs);
      cs.start();

    } catch (Exception e) {
      throw new OrchestraException("Problem registering mbean in mbean server",
          e);
    }
    try {

      EnvironmentFactory envFactory = BpelEnvironmentParser
          .parseEnvironmentFactoryFromResource(defaultEnvConfigFile);
      EnvFactoryRepository.set(envFactory);
    } catch (Exception e) {
      throw new OrchestraException("Problem creating the environment", e);
    }
    try {
      InputStream inputStream = this.getClass().getClassLoader()
          .getResourceAsStream("logging.properties");
      if (inputStream != null) {
        LogManager.getLogManager().readConfiguration(inputStream);
      }
    } catch (Exception e) {
      throw new OrchestraException("couldn't initialize logging properly", e);
    }
    Environment environment = null;
    try {
      environment = EnvFactoryRepository.get().openEnvironment();
      JobExecutor jobExecutor = environment.get(JobExecutor.class);
      if (jobExecutor != null) {
        jobExecutor.start();
      }
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
  }

  public void contextDestroyed(ServletContextEvent arg0) {
    Environment environment = null;
    try {
      environment = EnvFactoryRepository.get().openEnvironment();
      JobExecutor jobExecutor = environment.get(JobExecutor.class);
      if (jobExecutor != null) {
        jobExecutor.stop();
      }
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    EnvFactoryRepository.get().close();
    if (mbs != null) {
      try {
        cs.stop();
        mbs.unregisterMBean(objectName);
      } catch (Exception e) {
        throw new OrchestraException("Problem unregistering mbean", e);
      }
    }
  }

}
