package org.ow2.orchestra.jmx;

import java.io.InputStream;

/**
 * JMXConfiguration. Keeps all configuration parameters for the Jmx agent.
 *
 * @author Goulven Le Jeune, Charles Souillard, Candy Chlad, Stanislas Giraudet
 *         De Boudemange, Guillaume Porcher
 *
 */
public class JMXConfiguration {

  // Axis classes are run from WEB-INF/classes/, the jmx configuration file is
  // in WEB-INF/
  private static String jmxConfigFile = "orchestra-jmx-conf.xml";

  private static JMXConfiguration configuration;

  private int port = 9999;

  private String objectName = "JMXAgent:name=orchestraRemoteDeployer";

  private String serviceUrl = "service:jmx:rmi:///jndi/rmi://localhost:9999/orchestraServer";

  public static synchronized JMXConfiguration getInstance() {
    if (configuration == null) {
      ClassLoader classLoader = JMXConfiguration.class.getClassLoader();
      InputStream input = classLoader.getResourceAsStream(jmxConfigFile);
      configuration = new JMXConfiguration();
      new JMXConfigurationFileParser().parseAndSetObject(configuration, input);
    }
    return configuration;
  }

  /**
   * Returns the configuration parameters in a string.
   *
   * @return the string representing the configuration parameters
   */
  public String toString() {
    String s = "\n";
    s += "JMXConfiguration:" + "\n\t port: " + this.port + "\n\t objectName: "
        + this.objectName + "\n\t serviceUrl: " + this.serviceUrl + "\n";
    return s;
  }

  /**
   * @return the port
   */
  public int getPort() {
    return port;
  }

  /**
   * @param port
   *            the port to set
   */
  public void setPort(String port) {
    this.port = Integer.parseInt(port);
  }

  /**
   * @return the objectName
   */
  public String getObjectName() {
    return objectName;
  }

  /**
   * @param objectName
   *            the objectName to set
   */
  public void setObjectName(String objectName) {
    this.objectName = objectName;
  }

  /**
   * @return the serviceUrl
   */
  public String getServiceUrl() {
    return serviceUrl;
  }

  /**
   * @param serviceUrl
   *            the serviceUrl to set
   */
  public void setServiceUrl(String serviceUrl) {
    this.serviceUrl = serviceUrl;
  }

}
