/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx;

import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.jbpm.xml.Parse;
import org.ow2.orchestra.deployment.Deployment;

public interface RemoteDeployerMBean {

  /**
   * To deploy a process giving the bytes table of an archive file.<br>
   * The archive should contains the .bpel file and the .wsdl files.
   * @param barFile The bytes table of the bar file.
   * @return The qualified name of the deployed process.
   */
  QName deployBar(byte[] barFile);

  /**
   * To deploy the package giving the deployment object.
   * @param deployment The deployment object that should contain the bpel file and the wsdl definitions.
   * @return The qualified name of the deployed process.
   */
  QName deploy(Deployment deployment);

  /**
   * To undeploy the process giving the qualified name of the process.
   * @param processQName The qualified name.
   * @return true if the process has been succesfully undeployed otherwise
   * returns false.
   */
  boolean undeploy(QName processQName);

  /**
   * Returns the list of deployed processes.
   * @return The list of qualified name for deployed processes.
   */
  List<QName> listProcess();

  /**
   * Returns the status on created instances for all deployed processes.
   * @return The map containing these status.<br>
   * Key is the qualified name of the process.<br>
   * Value is the list of status for the instances of the process pointed by the key.
   */
  Map<QName, List<String>> listInstanceStatus();

  /**
   * Returns the informations on created instances for all deployed processes.
   * @return The map containing these informations.<br>
   * Key is the qualified name of the process.<br>
   * Value is the list of {@link InstanceInfo objects} containing the information for the instances of the
   * process pointed by the key.
   */
  Map<QName, List<InstanceInfo>> listInstance();

  /**
   * Returns the informations on the specified process.
   * @param qName The qualified name of the process.
   * @return the {@link ProcessInfo object} containing the information for the process.
   */
  ProcessInfo getProcess(QName qName);

  void initialize();
}
