/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the JBPM BPEL PUBLIC LICENSE AGREEMENT as
 * published by JBoss Inc.; either version 1.0 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package org.ow2.orchestra.lang;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.lang.evaluator.EvaluatorFactory;
import org.ow2.orchestra.util.Namespace;
import org.ow2.orchestra.util.XmlConstants;


/**
 * Common base for {@linkplain Expression expressions} and {@linkplain PropertyQuery queries}.
 * @author Alejandro Guizar
 * @version $Revision: 1.8 $ $Date: 2007/09/12 23:20:15 $
 */
public abstract class Snippet implements Serializable {

  protected long dbid;
  protected String text;
  protected String language;
  protected Set<Namespace> namespaces;

  private static final long serialVersionUID = 1L;

  protected Snippet() {
  }

  /**
   * Gets the lexical representation of this expression.
   * @return the text form
   */
  public String getText() {
    return text;
  }

  /**
   * Sets the lexical representation of this expression.
   * @param text the text form
   */
  public void setText(String text) {
    this.text = text;
  }

  public String getLanguage() {
    return language;
  }

  public void setLanguage(String language) {
    this.language = language;
  }

  public Set<Namespace> getNamespaces() {  
    return namespaces;
  }

  public void setNamespaces(Set<Namespace> namespaces) {
    this.namespaces = namespaces;
  }

  public void setNamespaces(Map<String, String> namespaceMap) {
    HashSet<Namespace> namespaces = new HashSet<Namespace>();
    Iterator<Entry<String, String>> namespaceEntryIt = namespaceMap.entrySet().iterator();
    while (namespaceEntryIt.hasNext()) {
      Entry<String, String> namespaceEntry = namespaceEntryIt.next();
      namespaces.add(new Namespace(namespaceEntry.getKey(),
          namespaceEntry.getValue()));
    }
    setNamespaces(namespaces);
  }

  protected EvaluatorFactory getEvaluatorFactory() {
    // get language specific to this script
    String myLanguage = getLanguage();
    // no specific language, use XPath 1.0
    if (myLanguage == null) {
      myLanguage = XmlConstants.URN_XPATH_1_0;
    }
    // get factory associated to language
    EvaluatorFactory factory = EvaluatorFactory.getInstance(myLanguage);
    return factory;
  }

  public String toString() {
    return "Snippet, text = " + text;
  }
}
