/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the JBPM BPEL PUBLIC LICENSE AGREEMENT as
 * published by JBoss Inc.; either version 1.0 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package org.ow2.orchestra.lang.evaluator;

import javax.xml.datatype.Duration;
import javax.xml.datatype.XMLGregorianCalendar;

import org.ow2.orchestra.runtime.BpelExecution;

/**
 * @author Alejandro Guizar
 * @version $Revision: 1.3 $ $Date: 2007/09/12 23:20:15 $
 */
public interface ExpressionEvaluator {

  /**
   * Evaluates the underlying expression, using the given token as the context
   * information.
   */
  Object evaluate(BpelExecution bpelExecution);

  /**
   * Assigns a value to the location identified by the underlying expression,
   * using the given token as the context information.
   */
  void assign(BpelExecution bpelExecution, Object value);

  /**
   * Evaluates the underlying expression as a Boolean expression and return the
   * evaluated value as a boolean.
   *
   * @param bpelExecution
   * @return the evaluated value as a boolean.
   */
  boolean evaluateBoolean(BpelExecution bpelExecution);

  /**
   * Evaluates the underlying expression as a deadline expression and return the
   * evaluated value as an {@link XMLGregorianCalendar}.
   *
   * The expression must be a string that is valid values of xsd:date and xsd:dateTime
   * otherwise, an {@link InvalidExpressionValue} will be thrown.
   *
   * @param bpelExecution
   * @return the evaluated value as an XMLGregorianCalendar.
   * @throws InvalidExpressionValue
   * @see XMLGregorianCalendar
   */
  XMLGregorianCalendar evaluateDeadline(BpelExecution bpelExecution)
    throws InvalidExpressionValue;

  /**
   * Evaluates the underlying expression as a duration expression and return the
   * evaluated value as a {@link Duration}.
   *
   * The expression must be a string that is valid values of xsd:duration otherwise, an
   * {@link InvalidExpressionValue} will be thrown.
   *
   * @param bpelExecution
   * @return the evaluated value as a Duration.
   * @throws InvalidExpressionValue
   * @see Duration
   */
  Duration evaluateDuration(BpelExecution bpelExecution)
    throws InvalidExpressionValue;

  /**
   * Evaluates the underlying expression as an unsigned integer and return the
   * evaluated value as an int.
   *
   * The numeric value must be valid values of xsd:unsignedInt otherwise, an
   * {@link InvalidExpressionValue} will be thrown.
   *
   * @param bpelExecution
   * @return the evaluated value as an int.
   * @throws InvalidExpressionValue
   */
  int evaluateUnsignedInt(BpelExecution bpelExecution)
    throws InvalidExpressionValue;
}
