/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the JBPM BPEL PUBLIC LICENSE AGREEMENT as
 * published by JBoss Inc.; either version 1.0 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package org.ow2.orchestra.lang.jaxen;

import java.util.Collections;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jaxen.Context;
import org.jaxen.ContextSupport;
import org.jaxen.FunctionContext;
import org.jaxen.JaxenException;
import org.jaxen.expr.Expr;
import org.jaxen.expr.LocationPath;
import org.ow2.orchestra.exception.BpelFaultException;
import org.ow2.orchestra.lang.evaluator.PropertyQueryEvaluator;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.util.XmlConstants;
import org.w3c.dom.Node;


/**
 * Default evaluator for property queries.
 * @author Alejandro Guizar
 * @version $Revision: 1.1 $ $Date: 2007/09/12 23:20:16 $
 */
class XPathPropertyQueryEvaluator extends XPathEvaluator implements PropertyQueryEvaluator {

  /**
   * The jBPM configuration object name for the resource describing the function library available
   * to property queries.
   */

  private static final long serialVersionUID = 1L;

  private static final Log LOG = LogFactory.getLog(XPathPropertyQueryEvaluator.class);

  private static final FunctionContext FUNCTION_LIBRARY = readFunctionLibrary();

  XPathPropertyQueryEvaluator(String text) throws JaxenException {
    super(text);
  }

  public synchronized Node evaluate(Node contextNode, BpelExecution bpelExecution) {
    try {
      Context context = getContext(contextNode, bpelExecution);
      List nodeSet = selectNodes(context);
      return narrowToSingleNode(nodeSet);
    } catch (JaxenException e) {
      LOG.error("query evaluation failed", e);
      throw new BpelFaultException(XmlConstants.FAULT_SUB_LANGUAGE_EXECUTION);
    }
  }

  public synchronized void assign(Node contextNode, Object value, BpelExecution bpelExecution) {
    Expr expr = getRootExpr();
    Context context = getContext(contextNode, bpelExecution);
    try {
      List nodeSet = expr instanceof LocationPath ? selectOrCreateNodes((LocationPath) expr,
          context) : selectNodes(context);
      BpelXmlUtil.setObjectValue(narrowToSingleNode(nodeSet), value, false);
    } catch (JaxenException e) {
      LOG.error("query assignment failed", e);
      throw new BpelFaultException(XmlConstants.FAULT_SUB_LANGUAGE_EXECUTION);
    }
  }

  protected FunctionContext createFunctionContext() {
    return FUNCTION_LIBRARY;
  }

  private Context getContext(Node contextNode, BpelExecution bpelExecution) {
    ContextSupport support = getContextSupport();
    support.setVariableContext(new BpelVariableContext(bpelExecution));

    Context context = new Context(support);
    context.setNodeSet(Collections.singletonList(contextNode));
    return context;
  }
}
