/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.xml.Parse;
import org.jbpm.xml.Parser;
import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.activity.ForEach;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.util.BpelUtil;
import org.w3c.dom.Element;

public class ForEachBinding extends ActivityBinding {

  /** log. */
  private static Logger log = Logger.getLogger(IfBinding.class.getName());

  public Object parse(Element element, Parse parse, Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    ForEach forEach = new ForEach(UUID.randomUUID().toString());
    org.jbpm.pvm.Node forEachNode = createActivityNode(element, parse, parser,
        forEach);

    setEnclosingScope(forEach, parse);
    parseStandardAttributes(element, forEach, parse);
    parseStandardElements(element, forEach, parse);

    parse.pushObject(forEachNode);
    parse.pushObject(forEach);
    try {
      BpelProcess bpelProcess = (BpelProcess) parse
          .findObject(BpelProcess.class);

      boolean parallel = BpelUtil.booleanValue(XmlUtil.attribute(element,
          "parallel"));
      if (parallel) {
        parse
            .addProblem("parallel feature not yet supported on forEach activity");
      }
      forEach.setParallel(parallel);
      forEach.setCounterName(XmlUtil.attribute(element, "counterName"));

      Element finalCounterValueElement = XmlUtil.element(element,
          "finalCounterValue");
      String finalCounterValueSt = finalCounterValueElement.getTextContent();
      String finalCounterValueLanguage = XmlUtil.attribute(
          finalCounterValueElement, "expressionLanguage");
      if (finalCounterValueLanguage == null) {
        finalCounterValueLanguage = bpelProcess.getExpressionLanguage();
      }
      Expression finalCounterValueExpression = new Expression();
      finalCounterValueExpression.setLanguage(finalCounterValueLanguage);
      finalCounterValueExpression.setNamespaces(BpelUtil
          .getAllNameSpaces(finalCounterValueElement));
      finalCounterValueExpression.setText(finalCounterValueSt);
      forEach.setFinalCounterValue(finalCounterValueExpression);

      Element startCounterValueElement = XmlUtil.element(element,
          "startCounterValue");
      String startCounterValueSt = startCounterValueElement.getTextContent();
      String startCounterValueLanguage = XmlUtil.attribute(
          startCounterValueElement, "expressionLanguage");
      if (startCounterValueLanguage == null) {
        startCounterValueLanguage = bpelProcess.getExpressionLanguage();
      }
      Expression startCounterValueExpression = new Expression();
      startCounterValueExpression.setLanguage(startCounterValueLanguage);
      startCounterValueExpression.setNamespaces(BpelUtil
          .getAllNameSpaces(startCounterValueElement));
      startCounterValueExpression.setText(startCounterValueSt);
      forEach.setStartCounterValue(startCounterValueExpression);

      Element completionConditionElement = XmlUtil.element(element,
          "completionCondition");
      if (completionConditionElement != null) {
        if (parallel) {
          parse
              .addProblem("parallel=yes with a completionCondition not yet supported on forEach activity");
        }
        Element branchesElement = XmlUtil.element(element, "branches");
        if (branchesElement == null) {
          parse
              .addProblem("Completion condition must contain at least one node from \"branches\"");
        } else {
          String completionConditionSt = branchesElement.getTextContent();
          String completionConditionLanguage = XmlUtil.attribute(
              branchesElement, "expressionLanguage");
          if (completionConditionLanguage == null) {
            completionConditionLanguage = bpelProcess.getExpressionLanguage();
          }
          Expression completionCondition = new Expression();
          startCounterValueExpression.setLanguage(completionConditionLanguage);
          startCounterValueExpression.setNamespaces(BpelUtil
              .getAllNameSpaces(branchesElement));
          startCounterValueExpression.setText(completionConditionSt);

          forEach.setSuccessfulBranchesOnly(BpelUtil.booleanValue(XmlUtil
              .attribute(branchesElement, "successfulBranchesOnly")));

          forEach.setCompletionCondition(completionCondition);
        }
      }

      parseChildActivity(element, parse, parser);
    } finally {
      parse.popObject();
      parse.popObject();
    }
    return forEachNode;
  }

}
