/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.xml.Parse;
import org.jbpm.xml.Parser;
import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.activity.If;
import org.ow2.orchestra.definition.element.Else;
import org.ow2.orchestra.definition.element.Elseif;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.util.BpelUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class IfBinding extends ActivityBinding {
  /** log. */
  private static Logger log = Logger.getLogger(IfBinding.class.getName());

  /**
   * Parses the given dom element to build a Node and a behaviour.
   * @param element dom element to parse
   * @param parse parse
   * @param parser parser
   * @return the build behaviour!
   */
  public Object parse(Element element, Parse parse, Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    If ifObject = new If(UUID.randomUUID().toString());
    org.jbpm.pvm.Node ifNode = createActivityNode(element, parse, parser, ifObject);

    setEnclosingScope(ifObject, parse);
    parseStandardAttributes(element, ifObject, parse);
    parseStandardElements(element, ifObject, parse);

    parse.pushObject(ifNode);
    parse.pushObject(ifObject);
    try {
      ifObject.setCondition(parseCondition(element, parse));
      ifObject.setActivity(parseChildActivity(element, parse, parser));

      // elseif elements
      List<Element> elseifElements = XmlUtil.elements(element, "elseif");
      parseElseifs(elseifElements, ifObject, parse, parser);

      //else element
      Element elseElement = XmlUtil.element(element, "else");
      parseElse(elseElement, ifObject, parse, parser);
    } finally {
      parse.popObject();
      parse.popObject();
    }
    return ifNode;
  }


  private void parseElseifs(List<Element> elseifElements, If ifObject, Parse parse, Parser parser) {
    List<Elseif> elseifs = new ArrayList<Elseif>();
    if (elseifElements != null) {
      for (Element elseifElement : elseifElements) {
        elseifs.add(parseElseif(elseifElement, parse, parser));
      }
    }
    ifObject.setElseifs(elseifs);
  }


  private Elseif parseElseif(Element elseifElement, Parse parse, Parser parser) {
    Elseif elseif = new Elseif();
    // condition
    elseif.setCondition(parseCondition(elseifElement, parse));
    elseif.setActivity(parseChildActivity(elseifElement, parse, parser));
    return elseif;
  }

  private void parseElse(Element elseElement, If ifObject, Parse parse, Parser parser) {
    if (elseElement != null) {
      Else else1 = new Else();
      else1.setActivity(parseChildActivity(elseElement, parse, parser));
      ifObject.setElse(else1);
    }
  }

  private Expression parseCondition(Element parentElement, Parse parse) {
    Element conditionElement = XmlUtil.element(parentElement, "condition");
    if (conditionElement != null) {
      BpelProcess bpelProcess = parse.findObject(BpelProcess.class);
      String expressionLanguage = XmlUtil.attribute(conditionElement, "expressionLanguage");
      if (expressionLanguage == null) {
        expressionLanguage = bpelProcess.getExpressionLanguage();
      }
      Expression expression = new Expression();
      expression.setLanguage(expressionLanguage);
      expression.setNamespaces(BpelUtil.getAllNameSpaces(conditionElement));
      expression.setText(conditionElement.getTextContent());
      return expression;
    }
    return null;
  }


}
