/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.xml.Parse;
import org.jbpm.xml.Parser;
import org.ow2.orchestra.definition.activity.Rethrow;
import org.ow2.orchestra.definition.element.Catch;
import org.w3c.dom.Element;

public class RethrowBinding extends ActivityBinding {

  /** log. */
  private static Logger log = Logger.getLogger(RethrowBinding.class.getName());

  /**
   * Parses the given dom element to build a Node and a behaviour.
   *
   * @param element
   *            dom element to parse
   * @param parse
   *            parse
   * @param parser
   *            parser
   * @return the build behaviour!
   */
  public Object parse(Element element, Parse parse, Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    Rethrow rethrow = new  Rethrow(UUID.randomUUID().toString());
    setEnclosingScope(rethrow, parse);
    parseStandardAttributes(element, rethrow, parse);
    parseStandardElements(element, rethrow, parse);

    Catch catch1 = parse.findObject(Catch.class);
    if (catch1 != null) {
      rethrow.setCathFaultName(catch1.getFaultName());
      rethrow.setCathVariableName(catch1.getFaultVariable().getName());
    }
    org.jbpm.pvm.Node activityNode = createActivityNode(element, parse, parser,
        rethrow);
    return activityNode;
  }

}
