/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.xml.Parse;
import org.jbpm.xml.Parser;
import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.activity.Wait;
import org.w3c.dom.Element;

/**
 * @author Pascal Verdage
 */
public class WaitBinding extends ActivityBinding {

  private static Logger log = Logger.getLogger(WaitBinding.class.getName());

  public Object parse(Element element, Parse parse, Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    Wait waitObject = new Wait(UUID.randomUUID().toString());
    org.jbpm.pvm.Node waitNode = createActivityNode(element, parse, parser, waitObject);

    setEnclosingScope(waitObject, parse);
    parseStandardAttributes(element, waitObject, parse);
    parseStandardElements(element, waitObject, parse);

    BpelProcess bpelProcess = parse.findObject(BpelProcess.class);
    Element deadlineElement = XmlUtil.element(element, "until");
    Element durationElement = XmlUtil.element(element, "for");
    if (durationElement != null && deadlineElement != null) {
      parse.addProblem("Error in element: " + element + " only one element 'for' or 'until' should be used.");
    }
    if (durationElement != null) {
      waitObject.setDurationExpression(PickBinding.parseExpression(durationElement, bpelProcess));
    } else if (deadlineElement != null) {
      waitObject.setDeadlineExpression(PickBinding.parseExpression(deadlineElement, bpelProcess));
    } else {
      parse.addProblem("Error in element: " + element + " one element 'for' or 'until' should be used.");
    }

    return waitNode;
  }

}
