/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.runtime;

import java.util.concurrent.atomic.AtomicInteger;

public class ForEachRuntime {

  // Should remain constant according to the spec!
  private final int startCounterValue;
  private final int finalCounterValue;
  private final AtomicInteger n;

  public ForEachRuntime(final int startCounterValue, final int finalCounterValue) {
    this.startCounterValue = startCounterValue;
    this.finalCounterValue = finalCounterValue;
    this.n = new AtomicInteger(finalCounterValue - startCounterValue + 1);
  }


  public int getStartCounterValue() {
    return startCounterValue;
  }


  public int getFinalCounterValue() {
    return finalCounterValue;
  }

  /**
   * Return true if the caller can perform another loop, false otherwise.
   *
   * Note: caller should loop again if the returned value is true, otherwise,
   * misbehavior will happen!
   *
   * @return
   */
  public boolean loopAgain() {
    return n.getAndDecrement() > 0;
  }

  public String toString() {
    return ForEachRuntime.class.getName()
      + "[startCounterValue: " + startCounterValue
      + ", finalCounterValue: " + finalCounterValue
      + ", n: " + n
      + "]";
  }
}
