/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import java.util.UUID;

import org.jbpm.env.Environment;
import org.jbpm.tx.StandardTransaction;
import org.jbpm.tx.Transaction;
import org.jbpm.util.Listener;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.var.Message;

public class MessageCarrier {

  private final Object lock = new Object();
  
  private Message message;

  private boolean hasMessage = false;

  private String uuid;

  public MessageCarrier() {
    this.uuid = UUID.randomUUID().toString();
  }

  public Message getMessage() {
    synchronized (lock) {
      while (!hasMessage) {
        try {
          lock.wait();
        } catch (InterruptedException e) {
          throw new OrchestraException(e);
        }
      }
      return message;
    }
  }

  public void setMessage(Message message) {
    synchronized (lock) {
      this.message = message.duplicate();
      hasMessage = true;
      Transaction tx = Environment.getCurrent().getTransaction();
      if (tx != null) {
        Listener listener = new Listener() {
          
          public void event(Object source, String eventName, Object info) {
            if (Transaction.EVENT_AFTERCOMPLETION.equals(eventName)
                && StandardTransaction.STATE_COMMITTED.equals(info)) {
              synchronized (MessageCarrier.this.lock) {
                MessageCarrier.this.lock.notifyAll();
              }
            }
          }
        };
        tx.addListener(listener, Transaction.EVENT_AFTERCOMPLETION);
      } else { 
        this.lock.notifyAll();
      }
    }
  }
  
  public synchronized boolean hasMessage() {
    synchronized (lock) {
      return hasMessage;  
    }
  }

  public String getUuid() {
    return uuid;
  }

}
