/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import java.util.HashMap;
import java.util.Map;

import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.var.Message;

public final class MessageCarrierRepository {

  private static Map<ReplierKey, MessageCarrier> messageCarriers = new HashMap<ReplierKey, MessageCarrier>();

  private static Map<String, MessageCarrier> pendingMessageCarriers = new HashMap<String, MessageCarrier>();

  private MessageCarrierRepository() {
  }

  public static synchronized void addMessageCarrier(ReplierKey replierKey,
      MessageCarrier messageCarrier) {
    messageCarriers.put(replierKey, messageCarrier);
  }

  public static synchronized void associateMessage(ReplierKey replierKey,
      Message message) {
    MessageCarrier messageCarrier = messageCarriers.remove(replierKey);
    if (messageCarrier == null) {
      throw new OrchestraException("No MessageCarrier corresponding to key = "
          + replierKey);
    }
    messageCarrier.setMessage(message);
  }

  public static synchronized void addPendingMessageCarrier(
      MessageCarrier messageCarrier) {
    pendingMessageCarriers.put(messageCarrier.getUuid(), messageCarrier);
  }

  public static synchronized MessageCarrier removePendingMessageCarrier(
      String messageCarrierId) {
    return pendingMessageCarriers.remove(messageCarrierId);
  }
}
