/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.orchestra.runtime.BpelExecution;

public final class MutexRepository {

  private static Map<QName, Object> processMutexs = new HashMap<QName, Object>();

  private static Map<String, Object> instanceMutexs = new HashMap<String, Object>();

  private MutexRepository() {
  }

  public static synchronized Object getProcessMutex(QName processQName) {
    Object mutex = processMutexs.get(processQName);
    if (mutex == null) {
      mutex = new Object();
      processMutexs.put(processQName, mutex);
    }
    return mutex;
  }

  public static synchronized void removeProcessMutex(QName processQName) {
    processMutexs.remove(processQName);
  }

  public static synchronized Object getInstanceMutex(String instanceUuid) {
    Object mutex = instanceMutexs.get(instanceUuid);
    if (mutex == null) {
      mutex = new Object();
      instanceMutexs.put(instanceUuid, mutex);
    }
    return mutex;
  }

  public static synchronized Object getInstanceMutex(BpelExecution bpelExecution) {
    BpelExecution instance = bpelExecution.getProcessInstance();
    return getInstanceMutex(instance.getUuid());
  }

  public static synchronized void removeInstanceMutex(String instanceUuid) {
    instanceMutexs.remove(instanceUuid);
  }

}
