/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import org.ow2.orchestra.runtime.ScopeRuntime;

public class ReplierKey {

  protected OperationKey operationKey;

  protected String messageExchange;

  protected ScopeRuntime scopeRuntime;

  protected ReplierKey() {
  }

  public ReplierKey(OperationKey operationKey, String messageExchange,
      ScopeRuntime scopeRuntime) {
    this.operationKey = operationKey;
    this.messageExchange = messageExchange;
    this.scopeRuntime = scopeRuntime;
  }

  @Override
  public int hashCode() {
    int hash = getOperationKey().hashCode() + 37 * getScopeRuntime().hashCode()
        + 37 * 37 * messageExchange.hashCode();
    return hash;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null || !(obj instanceof ReplierKey)) {
      return false;
    }
    ReplierKey other = (ReplierKey) obj;
    OperationKey otherOpKey = other.getOperationKey();
    return otherOpKey.equals(operationKey)
        && other.getMessageExchange().equals(messageExchange)
        && scopeRuntime.equals(other.getScopeRuntime());
  }

  public OperationKey getOperationKey() {
    return operationKey;
  }

  public String getMessageExchange() {
    return messageExchange;
  }

  public ScopeRuntime getScopeRuntime() {
    return scopeRuntime;
  }

  public String toString() {
    String st = "ReplierKey : OperationKey = " + operationKey.toString()
        + ", Scope = " + scopeRuntime.getUuid();
    if (messageExchange != null) {
      st += ", MessageExchange = " + messageExchange;
    }
    return st;
  }
}
