/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.db.hibernate;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.hibernate.usertype.UserType;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.util.BpelUtil;
import org.w3c.dom.Element;

/**
 * @author Juan Cantu
 * @version $Revision: 1.5 $ $Date: 2007/11/12 00:49:36 $
 */
public class ElementType implements UserType {

  private static final int[] SQL_TYPES = { Types.VARBINARY };

  public boolean equals(Object x, Object y) {
    if (x == null) {
      return y == null;
    }
    return x.equals(y);
  }

  public int hashCode(Object x) {
    return x.hashCode();
  }

  public Object deepCopy(Object value) {
    if (value != null) {
      return ((Element) value).cloneNode(true);
    }
    return null;
  }

  public boolean isMutable() {
    return true;
  }

  public Serializable disassemble(Object value) {
    return (Serializable) deepCopy(value);
  }

  public Object assemble(Serializable cached, Object owner) {
    return deepCopy(cached);
  }

  public Object replace(Object original, Object target, Object owner) {
    return deepCopy(original);
  }

  public int[] sqlTypes() {
    return SQL_TYPES;
  }

  public Object nullSafeGet(ResultSet rs, String[] names, Object owner) throws SQLException {
    // retrieve stream from database
    String columnName = names[0];
    InputStream xmlStream = null;

    try {
      xmlStream = rs.getBinaryStream(columnName);
      // if SQL value is NULL, element is null as well
      if (xmlStream == null) {
        return null;
      }
      // parse XML text
      Element element = BpelUtil.getDocumentFromInputStream(xmlStream).getDocumentElement();


      return element;
    } finally {
      if (xmlStream != null) {
        try {
          xmlStream.close();
        } catch (Exception e) {
          // nothing
        }
      }
    }
  }

  public void nullSafeSet(PreparedStatement st, Object value, int index) throws SQLException {
    // easy way out: null value
    if (value == null) {
      st.setNull(index, SQL_TYPES[0]);
    } else {
      Element element = (Element) value;
      try {
        ByteArrayOutputStream byteStream = BpelUtil.getNodeAsBAOS(element);
        st.setBytes(index, byteStream.toByteArray());
      } catch (OrchestraException e) {
        Class clazz = value.getClass();
        if (e.getCause() != null && e.getCause() instanceof ClassCastException
            && clazz.getName().startsWith("org.apache.axis.message.")) {
          st.setBytes(index, value.toString().getBytes());
        } else {
          throw e;
        }
      }
    }
  }

  public Class returnedClass() {
    return Element.class;
  }
}


