package org.ow2.orchestra.services.impl;

import java.util.List;

import javax.xml.namespace.QName;

import org.hibernate.FlushMode;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;
import org.jbpm.hibernate.HibernatePvmDbSession;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.Receiver;
import org.ow2.orchestra.services.ReplierKey;
import org.ow2.orchestra.services.db.hibernate.QNameUserType;
import org.ow2.orchestra.services.itf.BpelPersistenceService;
import org.ow2.orchestra.services.itf.Replier;

public class HibernateBpelPersistenceService extends
    HibernatePvmDbSession implements BpelPersistenceService {

  public HibernateBpelPersistenceService(Session session) {
    super(session);
    session.setFlushMode(FlushMode.COMMIT);
  }

  public BpelProcess findBpelProcess(String namespace, String name, int version) {
    Query query = getSession().getNamedQuery("findBpelProcess");
    query.setString("namespace", namespace);
    query.setString("name", name);
    query.setInteger("version", version);
    BpelProcess processDefinition = (BpelProcess) query.uniqueResult();
    return processDefinition;
  }

  public BpelProcess findBpelProcess(String namespace, String name) {
    Query query = getSession().getNamedQuery("findLatestBpelProcess");
    query.setString("namespace", namespace);
    query.setString("name", name);
    query.setMaxResults(1);
    BpelProcess processDefinition = (BpelProcess) query.uniqueResult();
    return processDefinition;
  }

  public List<BpelProcess> getBpelProcesses() {
    Query query = getSession().getNamedQuery("getBpelProcesses");
    return (List<BpelProcess>) query.list();
  }

  public Receiver getReceiver(OperationKey operationKey) {
    Query query = getSession().getNamedQuery("getReceiver");
    query.setParameter("processQName", operationKey.getProcessQName(),
        Hibernate.custom(QNameUserType.class));
    query.setParameter("portTypeQName", operationKey.getPortTypeQName(),
        Hibernate.custom(QNameUserType.class));
    query.setString("operationName", operationKey.getOperationName());
    query.setMaxResults(1);
    return (Receiver) query.uniqueResult();
  }

  public List<Receiver> getReceivers(QName processQName) {
    Query query = getSession().getNamedQuery("getReceivers");
    query.setParameter("processQName", processQName, Hibernate
        .custom(QNameUserType.class));
    return (List<Receiver>) query.list();
  }

  public Replier getReplier(ReplierKey replierKey) {
    Query query = getSession().getNamedQuery("getReplier");
    OperationKey operationKey = replierKey.getOperationKey();
    query.setParameter("processQName", operationKey.getProcessQName(),
        Hibernate.custom(QNameUserType.class));
    query.setParameter("portTypeQName", operationKey.getPortTypeQName(),
        Hibernate.custom(QNameUserType.class));
    query.setString("operationName", operationKey.getOperationName());
    query.setString("messageExchange", replierKey.getMessageExchange());
    query.setEntity("scope", replierKey.getScopeRuntime());
    query.setMaxResults(1);
    return (Replier) query.uniqueResult();
  }

  public List<BpelExecution> getBpelInstances() {
    Query query = getSession().getNamedQuery("getBpelInstances");
    return (List<BpelExecution>) query.list();
  }

  public List<BpelExecution> getBpelInstances(QName processQName) {
    Query query = getSession().getNamedQuery("getBpelInstancesOfProcess");
    query.setString("namespace", processQName.getNamespaceURI());
    query.setString("name", processQName.getLocalPart());
    return (List<BpelExecution>) query.list();
  }

  public List<Replier> getRepliers(ScopeRuntime scopeRuntime) {
    Query query = getSession().getNamedQuery("getRepliersByScopeRuntime");
    query.setEntity("scope", scopeRuntime);
    return query.list();
  }
  
}
