/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.impl;

import java.util.List;

import org.jbpm.env.Environment;
import org.jbpm.pvm.Activity;
import org.jbpm.pvm.Node;
import org.jbpm.pvm.impl.NodeImpl;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.element.InboundMessageElement;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.Receiver;
import org.ow2.orchestra.services.ReceivingElement;
import org.ow2.orchestra.services.itf.Publisher;
import org.ow2.orchestra.services.itf.Repository;

public class PublisherImpl implements Publisher {

  private Repository repository = null;

  private synchronized Repository getRepository() {
    if (repository == null) {
      repository = Environment.getCurrent().get(Repository.class);
    }
    return repository;
  }

  public Receiver getReceiver(OperationKey operationKey) {
    return getRepository().getReceiver(operationKey);
  }

  public Receiver removeReceiver(OperationKey operationKey) {
    return getRepository().removeReceiver(operationKey);
  }

  public void storeReceiver(OperationKey operationKey, Receiver receiver) {
    getRepository().storeReceiver(operationKey, receiver);
  }

  public void publishServices(BpelProcess bpelProcess, Environment environment) {
    publishServices(bpelProcess, (NodeImpl) bpelProcess.getInitial(),
        environment);
  }

  protected void publishServices(BpelProcess bpelProcess, NodeImpl node,
      Environment environment) {
    Activity behaviour = node.getBehaviour();
    if (behaviour instanceof InboundMessageElement) {
      InboundMessageElement inboundMessageElement = (InboundMessageElement) behaviour;
      for (ReceivingElement receivingElement : inboundMessageElement.getReceivingElements()) {
        OperationKey operationKey = receivingElement.getOperationKey(bpelProcess
            .getQName());
        Receiver receiver = createReceiverInstance(operationKey);
        storeReceiver(operationKey, receiver);
      }
    }
    List<Node> children = node.getNodes();
    if (children != null) {
      for (Node child : children) {
        publishServices(bpelProcess, (NodeImpl) child, environment);
      }
    }
  }

  public void unpublishServices(BpelProcess bpelProcess, Environment environment) {
    unpublishServices(bpelProcess, (NodeImpl) bpelProcess.getInitial(),
        environment);
  }

  public void unpublishServices(BpelProcess bpelProcess, NodeImpl node,
      Environment environment) {
    Activity behaviour = node.getBehaviour();
    if (behaviour instanceof InboundMessageElement) {
      InboundMessageElement inboundMessageElement = (InboundMessageElement) behaviour;
      for (ReceivingElement receivingElement : inboundMessageElement.getReceivingElements()) {
        OperationKey operationKey = receivingElement.getOperationKey(bpelProcess
            .getQName());
        removeReceiver(operationKey);
      }
    }
    List<Node> children = node.getNodes();
    if (children != null) {
      for (Node child : children) {
        unpublishServices(bpelProcess, (NodeImpl) child, environment);
      }
    }
  }

  public void republishServices(BpelProcess bpelProcess, Environment environment) {
  }

  protected Receiver createReceiverInstance(OperationKey operationKey) {
    return new Receiver(operationKey);
  }

}
