/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.util;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.ow2.orchestra.exception.OrchestraException;

/**
 * 
 * @author Charles Souillard
 */
public class StyleSheetRepository {

  /**
   * Store all the styleSheet used in the process. key: String =>name of the
   * styleSheet value: StreamSource => the parsed styleSheet
   */
  protected Map<String, URL> styleSheets;

  /**
   * @return the styleSheets
   */
  public Map<String, URL> getStyleSheets() {
    return styleSheets;
  }

  /**
   * @param styleSheets
   *            the styleSheets to set
   */
  public void setStyleSheets(Map<String, URL> styleSheets) {
    this.styleSheets = styleSheets;
  }

  /**
   * Add a new xslt styeSheet. The styleSheet is parsed and stored in the
   * styleSheet hashMap
   * 
   * @param fileAddress :
   *            address of the new styleSheet.
   */
  public void addStyleSheet(URL url, String styleSheetSt) {
    if (styleSheets == null) {
      styleSheets = new HashMap<String, URL>();
    }
    if (!styleSheets.containsValue(url)) {
      try {
        styleSheets.put(styleSheetSt, url);
      } catch (Exception e) {
        throw new OrchestraException(
            "Problem when creating inputSource from xsl url : " + url, e);
      }
    }
  }
}
