package org.ow2.orchestra.test;

import java.lang.reflect.Method;

import javax.xml.namespace.QName;

import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.PartnerLinkRuntime;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.itf.Invoker;
import org.ow2.orchestra.var.Message;

/**
 * @author Tom Baeyens, Charles Souillard
 */
public class TestInvoker implements Invoker {

  public Message invoke(OperationKey operationKey, String name,
      PartnerLinkRuntime partnerLinkRuntime, VariableRuntime variableRuntime)  {

    QName portTypeQName = operationKey.getPortTypeQName();
    Object instance = WSRepository.getWS(portTypeQName);
    if (instance == null) {
      throw new OrchestraException("No WS was found for portType : "
          + portTypeQName);
    }
    String operationName = operationKey.getOperationName();
    try {
      Method methodToCall = instance.getClass().getMethod(operationName,
          new Class[] { VariableRuntime.class });
      Object invokeReturn = methodToCall.invoke(instance,
          new Object[] { variableRuntime });
      if (invokeReturn != null) {
        if (!(invokeReturn instanceof Message)) {
          throw new OrchestraException("invokeReturn is not a message");
        }
        return (Message) invokeReturn;
      }
    } catch (Exception e) {
      throw new OrchestraException("problem invoking method : " + operationName
          + " : " + e.getMessage(), e);
    }

    return null;
  }

}
