package org.ow2.orchestra.test;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.PartnerLinkRuntime;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.services.itf.TestValidator;

/**
 * @author Tom Baeyens, Charles Souillard
 */
public class TestValidatorImpl implements TestValidator {

  private Map<BpelExecution, InstanceRecord> records = new HashMap<BpelExecution, InstanceRecord>();

  private static class InstanceRecord {

    protected Map<QName, List<VariableRuntime>> variables = new HashMap<QName, List<VariableRuntime>>();

    protected Map<QName, List<PartnerLinkRuntime>> partnerLinks = new HashMap<QName, List<PartnerLinkRuntime>>();

    protected Map<QName, List<String>> events = new HashMap<QName, List<String>>();
  }

  public void recordVariable(BpelExecution instance, QName processQName,
      VariableRuntime variableRuntime) {
    InstanceRecord instanceRecord = records.get(instance);
    if (instanceRecord == null) {
      records.put(instance, new InstanceRecord());
      instanceRecord = records.get(instance);
    }
    List<VariableRuntime> instanceVariables = instanceRecord.variables
        .get(processQName);
    if (instanceVariables == null) {
      instanceRecord.variables.put(processQName,
          new ArrayList<VariableRuntime>());
      instanceVariables = instanceRecord.variables.get(processQName);
    }
    instanceVariables.add(variableRuntime);
  }

  public List<VariableRuntime> getRecordedVariables(BpelExecution instance,
      QName processQName) {
    InstanceRecord instanceRecord = records.get(instance);
    if (instanceRecord != null) {
      return instanceRecord.variables.get(processQName);
    }
    return null;
  }

  public void recordPartnerLink(BpelExecution instance, QName processQName,
      PartnerLinkRuntime partnerLinkRuntime) {
    InstanceRecord instanceRecord = records.get(instance);
    if (instanceRecord == null) {
      records.put(instance, new InstanceRecord());
      instanceRecord = records.get(instance);
    }
    List<PartnerLinkRuntime> instancePartnerlinks = instanceRecord.partnerLinks
        .get(processQName);
    if (instancePartnerlinks == null) {
      instanceRecord.partnerLinks.put(processQName,
          new ArrayList<PartnerLinkRuntime>());
      instancePartnerlinks = instanceRecord.partnerLinks.get(processQName);
    }
    instancePartnerlinks.add(partnerLinkRuntime);
  }

  public List<PartnerLinkRuntime> getRecordedPartnerLinks(
      BpelExecution instance, QName processQName) {
    InstanceRecord instanceRecord = records.get(instance);
    if (instanceRecord != null) {
      return instanceRecord.partnerLinks.get(processQName);
    }
    return null;
  }

  public void recordEvent(BpelExecution instance, QName processQName,
      String event) {
    InstanceRecord instanceRecord = records.get(instance);
    if (instanceRecord == null) {
      records.put(instance, new InstanceRecord());
      instanceRecord = records.get(instance);
    }
    List<String> instanceEventss = instanceRecord.events.get(processQName);
    if (instanceEventss == null) {
      instanceRecord.events.put(processQName, new ArrayList<String>());
      instanceEventss = instanceRecord.events.get(processQName);
    }
    instanceEventss.add(event);
  }

  public List<String> getRecordedEvents(BpelExecution instance,
      QName processQName) {
    InstanceRecord instanceRecord = records.get(instance);
    if (instanceRecord != null) {
      return instanceRecord.events.get(processQName);
    }
    return null;
  }

  public void clean(BpelExecution instance) {
    records.remove(instance);
  }
}
