package org.ow2.orchestra.test.activities.foreach;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.util.XmlConstants;
import org.w3c.dom.Element;


public class ForEachTest extends BpelTestCase {
  /**
   * Default constructor.
   */
  public ForEachTest() {
    super("http://orchestra.ow2.org/foreach", "foreach");
  }

  public void testForEach() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    int start = 1;
    int stop = 4;
    int increment = 1;
    
    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("start", BpelXmlUtil.createElementWithContent(Integer.toString(start)));
    parts.put("stop", BpelXmlUtil.createElementWithContent(Integer.toString(stop)));
    parts.put("increment", BpelXmlUtil.createElementWithContent(Integer.toString(increment)));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName() + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      System.out.println(callResult);
      assertNotNull(callResult.getVariables());
      assertEquals(callResult.getVariables().size(), 1);

      Object nbEnd = getVariableValue(callResult, "nb");
      
      assertInt(nbEnd, (stop - start + 1) * increment);
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  private void assertInt(Object o, int expected) {
    assertTrue("int is not an instance of Element", o instanceof Element);
    Element defaultElement = (Element) o;
    assertEquals(XmlConstants.ELT_DEFAULT_NS, defaultElement.getNamespaceURI());
    assertEquals(XmlConstants.ELT_DEFAULT_ELEMENT, defaultElement
        .getLocalName());
    assertEquals(expected, new Integer(defaultElement.getTextContent())
        .intValue());
  }
}
