/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.invoke;

import java.net.URL;
import java.util.HashMap;
import java.util.List;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.test.wsutils.PersonWS;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class InvokeAddPersonTest extends BpelTestCase {

  public InvokeAddPersonTest() {
    super("http://example.com/invoke", "invokeAddPerson");
  }

  public void testInvokeAddPerson() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));

    URL personsList = this.getClass().getResource("personsList.xml");
    assertExists(personsList);
    URL person = this.getClass().getResource("person.xml");
    assertExists(person);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("personsList", BpelUtil.getDocumentFromURL(personsList)
        .getDocumentElement());
    parts.put("personToAdd", BpelUtil.getDocumentFromURL(person)
        .getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "invokeAddPerson";

    WSRepository.addWS(PersonWS.PORTTYPE_QNAME, new PersonWS());
    CallResult result = call(parts, portTypeQName, operationName);
    WSRepository.removeWS(PersonWS.PORTTYPE_QNAME);

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = result.getMessageCarrier().getMessage();
      String ns = PersonWS.PORTTYPE_QNAME.getNamespaceURI();
      assertNotNull(reply);
      Element element = reply.getPartValue("personsList");
      assertEquals(ns, element.getNamespaceURI());
      assertEquals("list", element.getLocalName());
      List<Element> children = BpelUtil.getElements(element);

      assertEquals(3, children.size());
      InvokeTest.assertPerson(children.get(0), "Paul", "Glood", "21",
          "Jules Vernes", "75000", "Paris");
      InvokeTest.assertPerson(children.get(1), "Roger", "Badin", "58",
          "Jean-Jacques ROUSSEAU", "13000", "Marseille");
      InvokeTest.assertPerson(children.get(2), "Arthur", "Monulo", "5",
          "5 Avenue", "4444", "Los Angeles");
      deleteInstance(result);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    undeploy();
  }

}
