/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.invoke;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.test.wsutils.PersonWS;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Charles Souillard
 */
public class InvokeCreatePersonTest extends BpelTestCase {

  public InvokeCreatePersonTest() {
    super("http://example.com/invoke", "invokePerson");
  }

  public void testInvokeCreatePerson() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));

    String firstName = "myFirstName";
    String lastName = "myLastName";
    String age = "26";
    String street = "myStreet";
    String zip = "38130";
    String city = "myCity";

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("firstName", BpelXmlUtil.createElementWithContent(firstName));
    parts.put("lastName", BpelXmlUtil.createElementWithContent(lastName));
    parts.put("age", BpelXmlUtil.createElementWithContent(age));
    parts.put("street", BpelXmlUtil.createElementWithContent(street));
    parts.put("zip", BpelXmlUtil.createElementWithContent(zip));
    parts.put("city", BpelXmlUtil.createElementWithContent(city));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "invokePerson";

    WSRepository.addWS(PersonWS.PORTTYPE_QNAME, new PersonWS());
    CallResult result = call(parts, portTypeQName, operationName);
    WSRepository.removeWS(PersonWS.PORTTYPE_QNAME);

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = result.getMessageCarrier().getMessage();
      String ns = PersonWS.PORTTYPE_QNAME.getNamespaceURI();
      assertNotNull(reply);
      Element personElement = reply.getPartValue("person");
      assertEquals("createPerson", personElement.getLocalName());
      assertEquals(ns, personElement.getNamespaceURI());
      NodeList children = personElement.getChildNodes();
      assertEquals(4, children.getLength());
      InvokeTest.assertElement(children.item(0), ns, "firstName", firstName);
      InvokeTest.assertElement(children.item(1), ns, "lastName", lastName);
      InvokeTest.assertElement(children.item(2), ns, "age", age);
      assertTrue(children.item(3) instanceof Element);
      Element addressEl = (Element) children.item(3);
      NodeList addressChildren = addressEl.getChildNodes();
      assertEquals(3, addressChildren.getLength());
      InvokeTest.assertElement(addressChildren.item(0), ns, "street", street);
      InvokeTest.assertElement(addressChildren.item(1), ns, "zip", zip);
      InvokeTest.assertElement(addressChildren.item(2), ns, "city", city);
      deleteInstance(result);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    undeploy();
  }

}
