/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.invoke;

import java.util.HashMap;
import java.util.List;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.test.wsutils.PersonWS;
import org.ow2.orchestra.test.wsutils.StringWS;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * @author Charles Souillard
 */
public class InvokeTest extends BpelTestCase {

  public InvokeTest() {
    super("http://example.com/invoke", "invoke");
  }

  public void testInvoke() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));

    String suffix = "-response";
    String content = "invoke input";

    Element inputElement = BpelUtil.getDocumentWithOneElement(new QName(
        "http://example.com/invoke", "invoke"));
    Text textContent = inputElement.getOwnerDocument().createTextNode(content);
    inputElement.appendChild(textContent);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("input", inputElement);

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "invoke";

    WSRepository.addWS(StringWS.PORTTYPE_QNAME, new StringWS(suffix));
    CallResult result = call(parts, portTypeQName, operationName);
    WSRepository.removeWS(StringWS.PORTTYPE_QNAME);

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = result.getMessageCarrier().getMessage();
      assertNotNull(reply);
      Element suffixElement = reply.getPartValue("output");
      assertEquals("invokeResponse", suffixElement.getLocalName());
      assertEquals("http://example.com/invoke", suffixElement.getNamespaceURI());
      assertTrue("output part root element first child is not a text",
          suffixElement.getFirstChild() instanceof Text);
      Text text = (Text) suffixElement.getFirstChild();
      assertEquals(content + suffix, text.getTextContent());
      deleteInstance(result);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    undeploy();

  }

  public static void assertElement(Node n, String ns, String localName,
      String textContent) {
    assertNotNull(n);
    assertTrue(n instanceof Element);
    Element el = (Element) n;
    assertEquals(ns, el.getNamespaceURI());
    assertEquals(localName, el.getLocalName());
    assertEquals(1, el.getChildNodes().getLength());
    Node child = el.getChildNodes().item(0);
    assertTrue(child instanceof Text);
    Text text = (Text) child;
    assertEquals(textContent, text.getTextContent());
  }

  public static void assertPerson(Node n, String firstName, String lastName,
      String age, String street, String zip, String city) {
    assertNotNull(n);
    assertTrue(n instanceof Element);
    Element el = (Element) n;
    String ns = PersonWS.PORTTYPE_QNAME.getNamespaceURI();
    assertEquals(ns, el.getNamespaceURI());
    assertEquals("person", el.getLocalName());

    List<Element> children = BpelUtil.getElements(el);

    assertEquals(4, children.size());
    assertElement(children.get(0), ns, "firstName", firstName);
    assertElement(children.get(1), ns, "lastName", lastName);
    assertElement(children.get(2), ns, "age", age);
    assertTrue(children.get(3) instanceof Element);
    Element addressEl = (Element) children.get(3);
    List<Element> addressChildren = BpelUtil.getElements(addressEl);
    assertEquals(3, addressChildren.size());
    assertElement(addressChildren.get(0), ns, "street", street);
    assertElement(addressChildren.get(1), ns, "zip", zip);
    assertElement(addressChildren.get(2), ns, "city", city);

  }
}
