/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have pickd a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.pick;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class Pick2Test extends BpelTestCase {

  /**
   * Default constructor.
   */
  public Pick2Test() {
    super("http://example.com/pick2", "pick2");
  }

  public void testPick2() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");

    HashMap<String, Element> parts = new HashMap<String, Element>();

    parts.put("st", BpelXmlUtil.createElementWithContent("pick1"));
    CallResult callResult1 = call(parts, portTypeQName, "op1");
    parts.put("st", BpelXmlUtil.createElementWithContent("pick2"));
    CallResult callResult2 = call(parts, portTypeQName, "op2");
    
    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();

      assertNotNull(callResult1.getVariables());
      assertEquals(1, callResult1.getVariables().size());

      assertNotNull(getVariableValue(callResult1, "st1"));
      
      assertNotNull(callResult2.getVariables());
      assertEquals(1, callResult2.getVariables().size());

      assertNotNull(getVariableValue(callResult2, "st2"));

      deleteInstance(callResult1);
      deleteInstance(callResult2);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }
}
