/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.pick;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.jbpm.jobexecutor.JobExecutor;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Pascal Verdage, Guillaume Porcher
 */
public class PickOnAlarmTest extends BpelTestCase {

  public PickOnAlarmTest() {
    super("http://example.com/pickOnAlarm", "pickOnAlarm");
  }

  public void setUp() {
    try {
      getEnvironmentFactory().openEnvironment();
      JobExecutor jobExecutor = Environment.getCurrent().get(JobExecutor.class);
      if (jobExecutor != null) {
        jobExecutor.setIdleInterval(500000);
        jobExecutor.start();
      }
    } finally {
      if (Environment.getCurrent() != null) {
        Environment.getCurrent().close();
      }
    }
  }

  public void tearDown() {
    try {
      getEnvironmentFactory().openEnvironment();
      JobExecutor jobExecutor = Environment.getCurrent().get(JobExecutor.class);
      if (jobExecutor != null) {
        jobExecutor.stop();
      }
    } finally {
      if (Environment.getCurrent() != null) {
        Environment.getCurrent().close();
      }
    }
  }
  
  public void testPickOnAlarm() {
    deploy();
    launch(2);
    launch(4);
    launch(6);
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"),
        this.getClass().getResource(getProcessName() + ".wsdl"));
  }

  public void launch(long durationInSec) {
    HashMap<String, Element> parts = new HashMap<String, Element>();

    parts.put("duration", BpelXmlUtil.createElementWithContent("PT" + durationInSec + "S"));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    
    long startTime = System.currentTimeMillis();
    long endTime = 0;
    CallResult callResult = call(parts, portTypeQName, operationName);

    try {
      getEnvironmentFactory().openEnvironment();
      Message message = callResult.getMessageCarrier().getMessage();
      Element durationElement = message.getPartValue("duration");
      
      assertNoElementChild(durationElement);
      
      long result = Long.parseLong(durationElement.getTextContent());
      endTime = System.currentTimeMillis();
      
      if (durationInSec < 4) {
        assertEquals(0, result);
        assertTrue(startTime + 1000 * durationInSec <= endTime);
      } else if (durationInSec > 4) {
        assertEquals(1, result);
        assertTrue(startTime + 4000 <= endTime);
      } else {
        if (result != 0 && result != 1) {
          fail("expected 0 or 1 but got " + result);
        }
      }

    
      getEnvironmentFactory().openEnvironment();
      deleteInstance(callResult);
    } finally {
      if (Environment.getCurrent() != null) {
        Environment.getCurrent().close();
      }
    }
  }

}
