/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have pickd a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.pick;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Pascal Verdage
 */
public class PickSessionTest extends BpelTestCase {

  
  public PickSessionTest() {
    super("http://example.com/pickSession", "pickSession");
  }

  public void testPickSession() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public void launch() {
    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");

    HashMap<String, Element> parts1 = new HashMap<String, Element>();
    parts1.put("id", BpelXmlUtil.createElementWithContent("1"));
    CallResult callResult1 = call(parts1, portTypeQName, "init");

    HashMap<String, Element> parts2 = new HashMap<String, Element>();
    parts2.put("id", BpelXmlUtil.createElementWithContent("2"));
    CallResult callResult2 = call(parts2, portTypeQName, "init");

    call(parts1, portTypeQName, "incrInt1");
    call(parts1, portTypeQName, "incrInt2");
    call(parts2, portTypeQName, "incrInt2");
    call(parts2, portTypeQName, "incrInt1");
    call(parts1, portTypeQName, "incrInt1");
    call(parts1, portTypeQName, "incrInt1");
    call(parts2, portTypeQName, "incrInt2");
    call(parts2, portTypeQName, "incrInt2");
    call(parts2, portTypeQName, "validate");
    call(parts1, portTypeQName, "incrInt1");
    call(parts1, portTypeQName, "incrInt2");
    call(parts1, portTypeQName, "validate");

    //the following calls should have no effect
    call(parts1, portTypeQName, "incrInt2");
    call(parts2, portTypeQName, "incrInt1");
    call(parts2, portTypeQName, "incrInt2");
    Environment environment = null;
    int instance1int1 = 0;
    int instance1int2 = 0;
    int instance2int1 = 0;
    int instance2int2 = 0;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply1 = callResult1.getMessageCarrier().getMessage();
      instance1int1 = Integer.parseInt(reply1.getPartValue("int1").getTextContent());
      instance1int2 = Integer.parseInt(reply1.getPartValue("int2").getTextContent());
      Message reply2 = callResult2.getMessageCarrier().getMessage();
      instance2int1 = Integer.parseInt(reply2.getPartValue("int1").getTextContent());
      instance2int2 = Integer.parseInt(reply2.getPartValue("int2").getTextContent());
      deleteInstance(callResult1);
      deleteInstance(callResult2);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    assertEquals(4, instance1int1);
    assertEquals(2, instance1int2);
    assertEquals(1, instance2int1);
    assertEquals(3, instance2int2);
  }

}
