/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.receive;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class Receive2Test extends BpelTestCase {

  /**
   * Default constructor.
   */
  public Receive2Test() {
    super("http://example.com/receive2", "receive2");
  }

  public void testReceive2() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");

    HashMap<String, Element> parts = new HashMap<String, Element>();

    parts.put("st", BpelXmlUtil.createElementWithContent("receive1"));
    CallResult callResult1 = call(parts, portTypeQName, "op1");

    parts.put("st", BpelXmlUtil.createElementWithContent("receive2"));
    CallResult callResult2 = call(parts, portTypeQName, "op2");

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();

      Message response1 = callResult1.getMessageCarrier().getMessage();
      Message response2 = callResult2.getMessageCarrier().getMessage();

      assertStPart(response1, "receive1");
      assertStPart(response2, "receive2");

      deleteInstance(callResult1);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertStPart(Message m, String expected) {
    Element defaultElement = (Element) m.getPartValue("out");
    assertEquals("http://example.com/receive2", defaultElement
        .getNamespaceURI());
    assertEquals("response", defaultElement.getLocalName());
    assertEquals(expected, defaultElement.getTextContent());
  }

}
