/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.receive;

import java.net.URL;
import java.util.HashMap;
import java.util.List;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.util.XmlConstants;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author Charles Souillard
 */
public class ReceiveTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public ReceiveTest() {
    super("http://example.com/receive", "receive");
  }

  public void testReceive() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL bigProduct = this.getClass().getResource("bigProduct.xml");
    assertExists(bigProduct);
    URL stElement = this.getClass().getResource("stElement.xml");
    assertExists(stElement);
    URL intElement = this.getClass().getResource("intElement.xml");
    assertExists(intElement);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("product", BpelUtil.getDocumentFromURL(bigProduct)
        .getDocumentElement());
    parts.put("stElement", BpelUtil.getDocumentFromURL(stElement)
        .getDocumentElement());
    parts.put("stType", BpelXmlUtil.createElementWithContent("Dupont"));
    parts.put("intElement", BpelUtil.getDocumentFromURL(intElement)
        .getDocumentElement());
    parts.put("intType", BpelXmlUtil.createElementWithContent("5"));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult result = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(result.getVariables());
      assertEquals(6, result.getVariables().size());

      assertProduct(getVariableValue(result, "product"));
      assertStType(getVariableValue(result, "stType"));
      assertStElement(getVariableValue(result, "stElement"));
      assertIntType(getVariableValue(result, "intType"));
      assertIntElement(getVariableValue(result, "intElement"));
      assertMessage(getVariableValue(result, "request"));
      deleteInstance(result);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertStType(Object o) {
    assertTrue("stType is not an instance of Element", o instanceof Element);
    Element defaultElement = (Element) o;
    assertEquals(XmlConstants.ELT_DEFAULT_NS, defaultElement.getNamespaceURI());
    assertEquals(XmlConstants.ELT_DEFAULT_ELEMENT, defaultElement
        .getLocalName());
    assertEquals("Dupont", defaultElement.getTextContent());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertStElement(Object o) {
    assertTrue("stElement is not an instance of Element", o instanceof Element);
    Element stElement = assertTopElementNS(o, "string");
    assertEquals("stringElementContent", stElement.getTextContent());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertIntType(Object o) {
    assertTrue("intType is not an instance of Element", o instanceof Element);
    Element defaultElement = (Element) o;
    assertEquals(XmlConstants.ELT_DEFAULT_NS, defaultElement.getNamespaceURI());
    assertEquals(XmlConstants.ELT_DEFAULT_ELEMENT, defaultElement
        .getLocalName());
    assertEquals("5", defaultElement.getTextContent());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertIntElement(Object o) {
    assertTrue("intElement is not an instance of Element", o instanceof Element);
    Element intElement = assertTopElementNS(o, "int");
    assertEquals("6", intElement.getTextContent());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertMessage(Object o) {
    assertTrue("request is not an instance of Message", o instanceof Message);
    Message message = (Message) o;
    assertProduct(message.getPartValue("product"));
    assertStElement(message.getPartValue("stElement"));
    assertIntType(message.getPartValue("intType"));
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertProduct(Object o) {
    assertTrue("product is not an instance of Element", o instanceof Element);
    Element bigProduct = assertTopElementNS(o, "bigProduct");
    assertEquals("123", bigProduct.getAttribute("amount"));
    assertEquals("chips", bigProduct.getAttribute("reference"));
    assertElementNSContentAndNoChild(bigProduct, "name", "myProduct");
    assertElementNSContentAndNoChild(bigProduct, "quantity", "52");
    List<Element> myElementChildren = XmlUtil.elements(bigProduct,
        getProcessNamespace(), "myElement");
    assertEquals(1, myElementChildren.size());
    Node myElementNode = myElementChildren.get(0);
    assertTrue(myElementNode instanceof Element);
    Element myElement = (Element) myElementNode;
    assertElementNSContentAndNoChild(myElement, "element1.1.1", "HelloWorld");
    assertElementNSContentAndNoChild(myElement, "element1.2", "123");
  }

}
