/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.reply;

import java.net.URL;
import java.util.HashMap;
import java.util.List;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.test.wsutils.PersonWS;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * @author Charles Souillard
 */
public class ReplyTest extends BpelTestCase {

  public ReplyTest() {
    super("http://example.com/reply", "replyToParts");
  }

  public void testReplyToParts() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));

    URL personsList = this.getClass().getResource("personsList.xml");
    assertExists(personsList);
    URL person = this.getClass().getResource("person.xml");
    assertExists(person);

    String personNS = PersonWS.PORTTYPE_QNAME.getNamespaceURI();

    Element personRootElement = BpelXmlUtil.createDefaultElement();
    Document personDocument = personRootElement.getOwnerDocument();
    Document personFileDoc = BpelUtil.getDocumentFromURL(person);
    Node imported = personDocument.importNode(personFileDoc
        .getDocumentElement(), true);
    personRootElement.appendChild(imported);

    String firstName = "myFirstName";
    Element firstnameInputRootElement = BpelUtil
        .getDocumentWithOneElement(new QName(personNS, "firstName"));
    Text textContent = firstnameInputRootElement.getOwnerDocument()
        .createTextNode(firstName);
    firstnameInputRootElement.appendChild(textContent);
    String lastName = "myLastName";

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("personsList", BpelUtil.getDocumentFromURL(personsList)
        .getDocumentElement());
    parts.put("person", personRootElement);
    parts.put("firstName", firstnameInputRootElement);
    parts.put("lastName", BpelXmlUtil.createElementWithContent(lastName));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "replyToParts";

    WSRepository.addWS(PersonWS.PORTTYPE_QNAME, new PersonWS());
    CallResult result = call(parts, portTypeQName, operationName);
    WSRepository.removeWS(PersonWS.PORTTYPE_QNAME);

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = result.getMessageCarrier().getMessage();
      String ns = PersonWS.PORTTYPE_QNAME.getNamespaceURI();
      assertNotNull(reply);

      Element listElement = reply.getPartValue("personsList");
      assertEquals(ns, listElement.getNamespaceURI());
      assertEquals("list", listElement.getLocalName());
      List<Element> children = BpelUtil.getElements(listElement);

      assertEquals(2, children.size());
      assertPerson(children.get(0), "Paul", "Glood", "21", "Jules Vernes",
          "75000", "Paris");
      assertPerson(children.get(1), "Roger", "Badin", "58",
          "Jean-Jacques ROUSSEAU", "13000", "Marseille");

      Element personElement = reply.getPartValue("person");
      List<Element> personChildrenElements = BpelUtil
          .getElements(personElement);
      assertEquals(1, personChildrenElements.size());
      assertPerson(personChildrenElements.get(0), "Arthur", "Monulo", "5",
          "5 Avenue", "4444", "Los Angeles");

      Element firstNameEl = reply.getPartValue("firstName");
      assertEquals(personNS, firstNameEl.getNamespaceURI());
      assertEquals("firstName", firstNameEl.getLocalName());
      assertEquals(0, BpelUtil.getElements(firstNameEl).size());
      List<Text> firstNameChildren = BpelUtil.getTexts(firstNameEl);
      assertEquals(1, firstNameChildren.size());
      Text text = firstNameChildren.get(0);
      assertEquals(firstName, text.getTextContent());

      Element lastNameElement = reply.getPartValue("lastName");
      assertNotNull(lastNameElement);
      assertEquals(lastName, lastNameElement.getTextContent());
      deleteInstance(result);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    undeploy();
  }

  private void assertElement(Node n, String ns, String localName,
      String textContent) {
    assertNotNull(n);
    assertTrue(n instanceof Element);
    Element el = (Element) n;
    assertEquals(ns, el.getNamespaceURI());
    assertEquals(localName, el.getLocalName());
    assertEquals(1, el.getChildNodes().getLength());
    Node child = el.getChildNodes().item(0);
    assertTrue(child instanceof Text);
    Text text = (Text) child;
    assertEquals(textContent, text.getTextContent());
  }

  private void assertPerson(Node n, String firstName, String lastName,
      String age, String street, String zip, String city) {
    assertNotNull(n);
    assertTrue(n instanceof Element);
    Element el = (Element) n;
    String ns = PersonWS.PORTTYPE_QNAME.getNamespaceURI();
    assertEquals(ns, el.getNamespaceURI());
    assertEquals("person", el.getLocalName());

    List<Element> children = BpelUtil.getElements(el);

    assertEquals(4, children.size());
    assertElement(children.get(0), ns, "firstName", firstName);
    assertElement(children.get(1), ns, "lastName", lastName);
    assertElement(children.get(2), ns, "age", age);
    assertTrue(children.get(3) instanceof Element);
    Element addressEl = (Element) children.get(3);
    List<Element> addressChildren = BpelUtil.getElements(addressEl);
    assertEquals(3, addressChildren.size());
    assertElement(addressChildren.get(0), ns, "street", street);
    assertElement(addressChildren.get(1), ns, "zip", zip);
    assertElement(addressChildren.get(2), ns, "city", city);

  }
}
