/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.throw_;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.exception.Fault;
import org.ow2.orchestra.exception.faults.InvalidVariables;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.test.wsutils.FaultWS;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Nicolas SALATGE - eBM WebSourcing
 */
public class ThrowTest extends BpelTestCase {


  private static final String FAULTWS_NS = "http://petals.ow2.org/FaultWS/";


  private static final String FAULTWS_PT_NAME = "FaultWS";

  public ThrowTest() {
    super("http://petals.ow2.org/recovery", "recoveryBpel");
  }



  public void testRecoverSpecificUserFault() {
    deploy(this.getClass().getResource("recoveryBpelUser.bpel"), this
        .getClass().getResource("recoveryBpel.wsdl"));
    try {
      recover("0");
      fail("expecting fault");
    } catch (Fault e) {
      assertEquals(e.getFaults().size(),1);
      assertEquals(e.getFaults().get(0).getFaultName(),new QName("http://petals.ow2.org/recovery","MyBpelFault"));
      deleteInstances(1);
    }
    undeploy();
  }

  public void testRecoverBpelFault() {
    deploy(this.getClass().getResource("recoveryBpel.bpel"), this
        .getClass().getResource("recoveryBpel.wsdl"));
    try {
      recover("0");
      fail("expecting InvalidVariables");
    } catch (InvalidVariables e) {
      assertEquals(e.getClass(), InvalidVariables.class);
      deleteInstances(1);
    }
    undeploy();
  }


  public void recover(String msg) {
//  deploy(this.getClass().getResource("recoveryBpel.bpel"), this
//  .getClass().getResource("recoveryBpel.wsdl"));

    Element processElement = BpelUtil.getDocumentWithOneElement(new QName(getProcessNamespace(), "recoveryBpelRequest"));

    Element inputElement = processElement.getOwnerDocument().createElementNS(getProcessNamespace(), "input");
    inputElement.setTextContent(msg);


    processElement.appendChild(inputElement);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("payload", processElement);

    QName portTypeQName = new QName(getProcessNamespace(), "recoveryBpel");
    String operationName = "process";


    QName faultWSQN = new QName(FAULTWS_NS, FAULTWS_PT_NAME);


    WSRepository.addWS(faultWSQN, new FaultWS(FAULTWS_NS));

    CallResult result = call(parts, portTypeQName, operationName);

    WSRepository.removeWS(faultWSQN);

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = result.getMessageCarrier().getMessage();
      assertNotNull(reply);

      Element respElement = reply.getPartValue("payload");

      Element recoveryBpelResponse = (Element) respElement.getFirstChild();

      System.out.println("recoveryBpelResponse = " + XmlUtil.toString(recoveryBpelResponse));

      assertEquals("specific business 0\\nTechnical Fault: Recover\\nUnknown Exception: Recover!\\n",
          recoveryBpelResponse.getTextContent());



      deleteInstance(result);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    undeploy();

  }


}
