/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.wait;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.w3c.dom.Element;

/**
 * @author Pascal Verdage
 */
public class WaitForTest extends WaitTestCase {

  public WaitForTest() {
    super("http://example.com/waitFor", "waitFor");
  }

  public void testWait() {
    deploy();
    launch();
    undeploy();
  }

  public void launch() {
    HashMap<String, Element> parts = new HashMap<String, Element>();

    parts.put("duration", BpelXmlUtil.createElementWithContent("PT2S"));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    
    long startTime = System.currentTimeMillis();
    long endTime = 0;
    CallResult callResult = call(parts, portTypeQName, operationName);

    try {
      getEnvironmentFactory().openEnvironment();
      callResult.getMessageCarrier().getMessage();
    } finally {
      if (Environment.getCurrent() != null) {
        Environment.getCurrent().close();
      }
    }
    endTime = System.currentTimeMillis();
    assertTrue(startTime + 2000 <= endTime);
    try {
      getEnvironmentFactory().openEnvironment();
      deleteInstance(callResult);
    } finally {
      if (Environment.getCurrent() != null) {
        Environment.getCurrent().close();
      }
    }
  }

}
