/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.wait;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.w3c.dom.Element;

/**
 * @author Pascal Verdage
 */
public class WaitUntilTest extends WaitTestCase {

  public WaitUntilTest() {
    super("http://example.com/waitUntil", "waitUntil");
  }

  public void testWait() {
    deploy();
    launch();
    undeploy();
  }

  public void launch() {
    HashMap<String, Element> parts = new HashMap<String, Element>();

    Date date = new Date(System.currentTimeMillis() + 2000);
    String deadline = null;
    DatatypeFactory datatypeFactory;
    try {
      datatypeFactory = DatatypeFactory.newInstance();
      XMLGregorianCalendar xmlCalendar = datatypeFactory.newXMLGregorianCalendar();
      GregorianCalendar calendar = xmlCalendar.toGregorianCalendar();
      calendar.setTime(date);
      xmlCalendar = datatypeFactory.newXMLGregorianCalendar(calendar);
      deadline = xmlCalendar.toString();
    } catch (Exception exc) {
      exc.printStackTrace();
      fail();
    }
    
    assertNotNull(deadline);

    parts.put("deadline", BpelXmlUtil.createElementWithContent(deadline));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    
    long startTimeSecond = System.currentTimeMillis() / 1000;
    long endTimeSecond = 0;
    CallResult callResult = call(parts, portTypeQName, operationName);

    try {
      getEnvironmentFactory().openEnvironment();
      callResult.getMessageCarrier().getMessage();
    } finally {
      if (Environment.getCurrent() != null) {
        Environment.getCurrent().close();
      }
    }
    endTimeSecond = System.currentTimeMillis() / 1000;
    assertTrue(startTimeSecond + 2 <= endTimeSecond);
    try {
      getEnvironmentFactory().openEnvironment();
      deleteInstance(callResult);
    } finally {
      if (Environment.getCurrent() != null) {
        Environment.getCurrent().close();
      }
    }
  }

}
