/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.activities.while_;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.util.XmlConstants;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class WhileTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public WhileTest() {
    super("http://example.com/while", "while");
  }

  public void testWhile() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    int nb = 10;
    int loop = 85;

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("nb", BpelXmlUtil.createElementWithContent(Integer.toString(nb)));
    parts.put("loop", BpelXmlUtil.createElementWithContent(Integer
        .toString(loop)));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());
      assertEquals(callResult.getVariables().size(), 2);

      Object nbEnd = getVariableValue(callResult, "nb");
      Object loopEnd = getVariableValue(callResult, "loop");

      assertInt(nbEnd, nb + loop);
      assertInt(loopEnd, 0);
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  private void assertInt(Object o, int expected) {
    assertTrue("int is not an instance of Element", o instanceof Element);
    Element defaultElement = (Element) o;
    assertEquals(XmlConstants.ELT_DEFAULT_NS, defaultElement.getNamespaceURI());
    assertEquals(XmlConstants.ELT_DEFAULT_ELEMENT, defaultElement
        .getLocalName());
    assertEquals(expected, new Integer(defaultElement.getTextContent())
        .intValue());
  }

}
