/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.correlation.checkCorrelation;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.exception.CorrelationViolation;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.test.correlation.CorrelationTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class CheckCorrelationTest extends CorrelationTestCase {

  /**
   * Default constructor.
   */
  public CheckCorrelationTest() {
    super("http://example.com/checkCorrelation", "checkCorrelation");
  }

  public void testCheckCorrelationRequestResponse() {
    deploy(false);
    launch();
    undeploy();
  }
  
  public void testCheckCorrelationBadResponse() {
    deploy(true);
    try {
      launch();
      fail("CorrelationViolation expected");
    } catch (CorrelationViolation e) {
      deleteInstances(1);
    }
    undeploy();
  }
  
  public void testCheckCorrelationBadRequest() {
    deploy(this.getClass().getResource(getProcessName() + "2.bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
    WSRepository.addWS(CorrelationWS.PORTTYPE_QNAME, new CorrelationWS("Valid", false));
    try {
      launch();
      fail("CorrelationViolation expected");
    } catch (CorrelationViolation e) {
      deleteInstances(1);
    }
    undeploy();
  }



  public void deploy(boolean invokeChangeCS) {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
    WSRepository.addWS(CorrelationWS.PORTTYPE_QNAME, new CorrelationWS("Valid", invokeChangeCS));
  }

  public void undeploy() {
    WSRepository.removeWS(CorrelationWS.PORTTYPE_QNAME);
    super.undeploy();
  }
  
  public long launch() {
    String auctionId = "1";
    String shippingCosts = "1";
    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("shippingCosts", BpelXmlUtil
        .createElementWithContent(shippingCosts));
    parts.put("auctionId", BpelXmlUtil.createElementWithContent(auctionId));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());
      assertEquals(1, callResult.getVariables().size());

      String expected = auctionId + "-" + shippingCosts + "-" + auctionId
          + "-" + shippingCosts + "Valid";
      Object result = getVariableValue(callResult, "result");
      assertResult(result, expected);
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   * @param expected
   *            expected result
   */
  private void assertResult(Object o, String expected) {
    assertNotNull(o);
    assertTrue("result is not an instance of Element", o instanceof Element);
    Element result = (Element) o;
    assertEquals(expected, result.getTextContent());
  }

}
