/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.correlation.checkCorrelation;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class CorrelationWS {

  private String st;

  private boolean changeAuctionId;

  private static final String NAMESPACE = "http://orchestra.ow2.org/correlationWS";

  public static final QName PORTTYPE_QNAME = new QName(NAMESPACE, "correlationWSPT");

  public CorrelationWS(String st, boolean changeAuctionId) {
    this.st = st;
    this.changeAuctionId = changeAuctionId;
  }

  public Message suffix(VariableRuntime variableRuntime) {
    Object varValue = variableRuntime.getValue();
    if (!(varValue instanceof Message)) {
      throw new OrchestraException("variableRuntime value is not a message");
    }
    Message suffixRequest = (Message) varValue;
    Element suffixElement = suffixRequest.getPartValue("input");
    if (!(suffixElement.getFirstChild() instanceof Text)) {
      throw new OrchestraException("input part first child is not a text");
    }

    Text text = (Text) suffixElement.getFirstChild();
    String content = text.getTextContent();

    Element outputRootElement = BpelUtil.getDocumentWithOneElement(new QName(
        NAMESPACE, "suffix"));
    Text textContent = outputRootElement.getOwnerDocument().createTextNode(
        content + st);
    outputRootElement.appendChild(textContent);
    
    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("output", outputRootElement);
    if (!changeAuctionId) {
      parts.put("auctionId", suffixRequest.getPartValue("auctionId"));
    } else {
      Element auctionId = suffixRequest.getPartValue("auctionId");
      auctionId.setTextContent(auctionId.getTextContent() + "changed");
      parts.put("auctionId", auctionId);
    }
    
    return new Message(parts);
  }

}
