/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.correlation.replyCorrelation;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.exception.CorrelationViolation;
import org.ow2.orchestra.test.correlation.CorrelationTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Guillaume Porcher
 */
public class ReplyCorrelationTest extends CorrelationTestCase {

  public ReplyCorrelationTest() {
    super("http://orchestra.ow2.org/replyCorrelation", "replyCorrelation");
  }

  protected CallResult makeCall(String firstName, String lastName, Boolean change) {
    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("firstName", BpelXmlUtil.createElementWithContent(firstName));
    parts.put("lastName", BpelXmlUtil.createElementWithContent(lastName));
    parts.put("change", BpelXmlUtil.createElementWithContent(change.toString()));
    
    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "replyCorrelation";

    return call(parts, portTypeQName, operationName);
  }
  
  public void testReplyCorrelationOK() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
    String firstName = "myFirstName";
    String lastName = "myLastName";
    CallResult result = makeCall(firstName, lastName, false);
    
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = result.getMessageCarrier().getMessage();
      assertNotNull(reply);

      Element firstNameEl = reply.getPartValue("firstName");
      assertNotNull(firstNameEl);
      assertEquals(firstName, firstNameEl.getTextContent());
      
      Element lastNameElement = reply.getPartValue("lastName");
      assertNotNull(lastNameElement);
      assertEquals(lastName, lastNameElement.getTextContent());
      deleteInstance(result);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    undeploy();
  }

  public void testReplyBadCorrelation() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
    try {
      makeCall("firstName", "lastName", true);
      fail("Expected CorrelationViolation exception");
    } catch (CorrelationViolation e) {
      // OK
      deleteInstances(1);
    }
    undeploy();
  }

  
  public void testReplyCorrelationInitiate() {
    deploy(this.getClass().getResource(getProcessName() + "Initiate.bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
    try {
      makeCall("firstName", "lastName", false);
      fail("Expected CorrelationViolation exception");
    } catch (CorrelationViolation e) {
      // OK
      deleteInstances(1);
    }
    undeploy();
  }
 
}
