/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.correlation.simpleCorrelation;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class SimpleCorrelationEltTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public SimpleCorrelationEltTest() {
    super("http://example.com/simpleCorrelationElt", "simpleCorrelationElt");
  }

  public void testSimpleCorrelation() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    String auctionId = "1";
    String shippingCosts1 = "1";
    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("shippingCosts", BpelXmlUtil
        .createElementWithContent(shippingCosts1));
    Element auct1 = BpelXmlUtil.createElement(getProcessNamespace(), "t1:auctionId");
    auct1.setTextContent(auctionId);
    parts.put("auctionId", auct1);

    String shippingCosts2 = "2";
    HashMap<String, Element> parts2 = new HashMap<String, Element>();
    parts2.put("shippingCosts", BpelXmlUtil
        .createElementWithContent(shippingCosts2));
    Element auct2 = BpelXmlUtil.createElement(getProcessNamespace(), "t2:auctionId");
    auct2.setTextContent(auctionId);
    parts2.put("auctionId", auct2);

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    call(parts, portTypeQName, operationName);
    CallResult callResult = call(parts2, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());
      assertEquals(1, callResult.getVariables().size());

      String expected1 = auctionId + "-" + shippingCosts1 + "-" + auctionId
          + "-" + shippingCosts2;
      String expected2 = auctionId + "-" + shippingCosts2 + "-" + auctionId
          + "-" + shippingCosts1;
      Object result = getVariableValue(callResult, "result");
      if (!assertResult(result, expected1) && !assertResult(result, expected2)) {
        assertTrue("wrong correlation, expected was either " + expected1
            + ", either " + expected2 + "and the result is " + result, false);
      }
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   * @param expected
   *            expected result
   */
  private boolean assertResult(Object o, String expected) {
    assertNotNull(o);
    assertTrue("result is not an instance of Element", o instanceof Element);
    Element result = (Element) o;
    boolean b = expected.equals(result.getTextContent());
    return b;
  }

}
