/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.integration.correlation.sales;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Nicolas SALATGE - eBM WebSourcing
 */
public class SalesTest extends BpelTestCase {

  private static final String NS = "http://petals.ow2.org/sales";

  private static final String SALES_PT_NAME = "sales";
  private static final String STOCK_PT_NAME = "stock";

  public SalesTest() {
    super("http://petals.ow2.org/sales", "salesAsync");
  }

  public void testInititiate1Process() {
    process(4, true);
  }

  public void testInititiate2Process() {
    process(5, false);
  }

  public void process(int refId, boolean res) {
    deploy(this.getClass().getResource("sales.bpel"), this
        .getClass().getResource("sales.wsdl"));

    Element processElement = BpelUtil.getDocumentWithOneElement(new QName(getProcessNamespace(), "salesRequest"));

    Element itemElement = processElement.getOwnerDocument().createElementNS(getProcessNamespace(), "article");

    Element refElement = processElement.getOwnerDocument().createElementNS(getProcessNamespace(), "refId");
    refElement.setTextContent(String.valueOf(refId));

    Element nameElement = processElement.getOwnerDocument().createElementNS(getProcessNamespace(), "name");
    nameElement.setTextContent("shirt");

    Element quantityElement = processElement.getOwnerDocument().createElementNS(getProcessNamespace(), "quantity");
    quantityElement.setTextContent("4");


    itemElement.appendChild(refElement);
    itemElement.appendChild(nameElement);
    itemElement.appendChild(quantityElement);

    processElement.appendChild(itemElement);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("payload", processElement);

    QName portTypeQName = new QName(getProcessNamespace(), "sales");
    String operationName = "initiate";

    QName stockQN = new QName(NS, STOCK_PT_NAME);

    WSRepository.addWS(stockQN, new Stock());

    CallResult result = call(parts, portTypeQName, operationName);

    WSRepository.removeWS(stockQN);

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();

      Message reply = result.getMessageCarrier().getMessage();

      assertNotNull(reply);
      Element responseElement = reply.getPartValue("payload");

      assertNotNull(((Element)responseElement).getFirstChild());

      if (res) {
        assertEquals(((Element)responseElement).getFirstChild().getTextContent(), "OK: The command is registered");
      } else {
        assertEquals(((Element)responseElement).getFirstChild().getTextContent(), "Error: The quantity is too important");
      }
      deleteInstance(result);
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    undeploy();

  }
}
