/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.integration.evaluate;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Nicolas Salatge
 */
public class EvaluateTest extends BpelTestCase {

  private static final String WEATHER_NS = "http://petals.ow2.org";
  private static final String STOCK_NS = "http://petals.ow2.org";
  private static final String ANALYZE_NS = "http://petals.ow2.org";

  private static final String WEATHER_PT_NAME = "soapWeatherInterface";
  private static final String STOCK_PT_NAME = "soapStockInterface";
  private static final String ANALYZE_PT_NAME = "analyzer";

  public EvaluateTest() {
    super("http://petals.ow2.org/wsportal/", "wsPortalBPEL");
  }

  public void testWeatherProcess() {
    process("weather");
  }

  public void testStockProcess() {
    process("stock");
  }



  public void process(String op) {
    deploy(this.getClass().getResource("wsportal.bpel"), this
        .getClass().getResource("wsportal.wsdl"));

    Element processElement = BpelUtil.getDocumentWithOneElement(new QName(getProcessNamespace(), "process"));

    Element requestElement = processElement.getOwnerDocument().createElementNS(getProcessNamespace(), "request");

    requestElement.setTextContent("!" + op + " toulouse");

    processElement.appendChild(requestElement);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("request", requestElement);

    System.out.println("getProcessNamespace = " + getProcessNamespace());
    QName portTypeQName = new QName(getProcessNamespace(), "wsportal");
    String operationName = "process";

    QName analyzeQN = new QName(ANALYZE_NS, ANALYZE_PT_NAME);
    QName weatherQN = new QName(WEATHER_NS, WEATHER_PT_NAME);
    QName stockQN = new QName(STOCK_NS, STOCK_PT_NAME);

    WSRepository.addWS(analyzeQN, new Analyze(ANALYZE_NS, op));
    WSRepository.addWS(weatherQN, new Weather(WEATHER_NS));
    WSRepository.addWS(stockQN, new Stock(STOCK_NS));
    CallResult result = call(parts, portTypeQName, operationName);
    WSRepository.removeWS(analyzeQN);
    WSRepository.removeWS(weatherQN);
    WSRepository.removeWS(stockQN);

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = result.getMessageCarrier().getMessage();
      assertNotNull(reply);
      Element responseElement = reply.getPartValue("response");

      System.out.println("result = " + result.getMessageCarrier().getMessage());

      if (op.equals("weather")) {
        assertNotNull((Element)responseElement.getElementsByTagName("weatherReturn").item(0));
        assertEquals(((Element)responseElement.getElementsByTagName("weatherReturn").item(0)).getTextContent(), "18");
      } else {
        assertNotNull((Element)responseElement.getElementsByTagName("stockReturn").item(0));
        assertEquals(((Element)responseElement.getElementsByTagName("stockReturn").item(0)).getTextContent(), "18");
      }

      deleteInstance(result);
    } catch (Exception e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    undeploy();

  }
}
