/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.integration.sc;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class SCTest extends BpelTestCase {

  private static final String SA_NS = "http://petals.ow2.org/SA";
  private static final String SB_NS = "http://petals.ow2.org/SB";
  private static final String ANALYZER_NS = "http://petals.ow2.org/SC";

  private static final String SA_PT_NAME = "SAInterface";
  private static final String SB_PT_NAME = "SBInterface";
  private static final String ANALYZER_PT_NAME = "analyzerDocumentInterface";

  public SCTest() {
    super("http://petals.ow2.org/SC", "SC");
  }

  public void testGetAdresse() {
    getAdresse("13");
  }

  public void testGetAdresseNull() {
    getAdresse("1");
  }

  public void testListeGetAdresse() {
    getListeAdresse("route du puit");
  }

  public void getAdresse(String id) {
    deploy(this.getClass().getResource("SC.bpel"), this
        .getClass().getResource("SC.wsdl"));

    Element idElement = BpelUtil.getDocumentWithOneElement(new QName(getProcessNamespace(), "id"));
    idElement.setTextContent(id);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("IDAdresse", idElement);

    QName portTypeQName = new QName(getProcessNamespace(), "SCBPELInterface");
    String operationName = "getAdresse";

    QName saQN = new QName(SA_NS, SA_PT_NAME);
    QName sbQN = new QName(SB_NS, SB_PT_NAME);
    QName analyzerQN = new QName(ANALYZER_NS, ANALYZER_PT_NAME);

    WSRepository.addWS(saQN, new ABWS(SA_NS));
    WSRepository.addWS(sbQN, new ABWS(SB_NS));
    WSRepository.addWS(analyzerQN, new AnalyzerWS(ANALYZER_NS));

    CallResult result = call(parts, portTypeQName, operationName);
    WSRepository.removeWS(saQN);
    WSRepository.removeWS(sbQN);
    WSRepository.removeWS(analyzerQN);

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = result.getMessageCarrier().getMessage();
      assertNotNull(reply);

      System.out.println("reply = " + reply);

      Element addressElement = reply.getPartValue("address");

      assertEquals(addressElement.getChildNodes().getLength(), 5);


      deleteInstance(result);
    } finally {
      if (environment != null) {
        System.out.println("environment = " + environment);
        environment.close();
      }
    }
    undeploy();
  }

  public void getListeAdresse(String rue) {
    deploy(this.getClass().getResource("SC.bpel"), this
        .getClass().getResource("SC.wsdl"));

    Element rueElement = BpelUtil.getDocumentWithOneElement(new QName(getProcessNamespace(), "rue"));
    rueElement.setTextContent(rue);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("rue", rueElement);

    QName portTypeQName = new QName(getProcessNamespace(), "SCBPELInterface");
    String operationName = "getListeAdresses";

    QName saQN = new QName(SA_NS, SA_PT_NAME);
    QName sbQN = new QName(SB_NS, SB_PT_NAME);
    QName analyzerQN = new QName(ANALYZER_NS, ANALYZER_PT_NAME);

    WSRepository.addWS(saQN, new ABWS(SA_NS));
    WSRepository.addWS(sbQN, new ABWS(SB_NS));
    WSRepository.addWS(analyzerQN, new AnalyzerWS(ANALYZER_NS));

    CallResult result = call(parts, portTypeQName, operationName);
    WSRepository.removeWS(saQN);
    WSRepository.removeWS(sbQN);
    WSRepository.removeWS(analyzerQN);

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = result.getMessageCarrier().getMessage();
      assertNotNull(reply);
      Element addressElement = reply.getPartValue("getListAdressesResponse");

      assertEquals(addressElement.getChildNodes().getLength(), 4);
      assertEquals(addressElement.getFirstChild().getFirstChild().getTextContent(), "13");


      deleteInstance(result);
    } finally {
      if (environment != null) {
        System.out.println("environment = " + environment);
        environment.close();
      }
    }
    undeploy();

  }

}
