/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.partnerLink.partnerLink;

import java.net.URL;
import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.runtime.PartnerLinkRuntime;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class PartnerLinkTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public PartnerLinkTest() {
    super("http://example.com/partnerLink", "partnerLink");
  }

  public void testPartnerLink() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL epr1 = this.getClass().getResource("epr1.xml");
    assertExists(epr1);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("epr1", BpelUtil.getDocumentFromURL(epr1).getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getPartnerLInks());
      assertEquals(4, callResult.getPartnerLInks().size());

      assertNotNull(callResult.getVariables());
      assertEquals(10, callResult.getVariables().size());

      // from var partnerLink
      assertPartnerLink(getPartnerLink(callResult, "partnerLinkPL2"));
      assertPartnerLink(getPartnerLink(callResult, "partnerLinkPL3"));
      // assertPartnerLink(getPartnerLink(callResult, "partnerLinkPL4"));
      assertPartnerLink(getPartnerLink(callResult, "partnerLinkPL5"));

      assertMessage(getVariableValue(callResult, "request1"));
      assertMessage(getVariableValue(callResult, "request2"));
      assertMessage(getVariableValue(callResult, "request3"));
      assertMessage(getVariableValue(callResult, "request4"));
      assertMessage(getVariableValue(callResult, "request5"));

      assertEprVariable(getVariableValue(callResult, "epr1"));
      assertEprVariable(getVariableValue(callResult, "epr2"));
      assertEprVariable(getVariableValue(callResult, "epr3"));
      assertEprVariable(getVariableValue(callResult, "epr4"));
      assertEprVariable(getVariableValue(callResult, "epr5"));
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param partnerLink
   *            object to assert.
   */
  private void assertPartnerLink(PartnerLinkRuntime partnerLink) {
    assertNotNull(partnerLink);
    assertServiceRefElement(partnerLink.getPartnerRoleEndPointReference());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertServiceRefElement(Object o) {
    assertServiceRefElement(o, "http://localhost:8080/partnerLinkService",
        new QName(getProcessNamespace(), "partnerLinkService"));
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertServiceRefType(Object o) {
    assertServiceRefType(o, "http://localhost:8080/partnerLinkService",
        new QName(getProcessNamespace(), "partnerLinkService"));
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertEprVariable(Object o) {
    assertServiceRefType(o);
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertMessage(Object o) {
    assertNotNull(o);
    assertTrue("o is not an instance of Message", o instanceof Message);
    Element epr1 = ((Message) o).getPartValue("epr1");
    assertNotNull(epr1);
    assertServiceRefType(epr1);
    Element epr2 = ((Message) o).getPartValue("epr2");
    assertNotNull(epr2);
    assertServiceRefElement(epr2);
  }

}
