/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.perf.flow;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.test.perf.AbstractPerfTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class FlowTest extends AbstractPerfTestCase {

  public FlowTest() {
    super("http://example.com/flow", "flow");
  }

  /**
   * Deploys the process.
   */
  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  /**
   * Launches an instance of the process.
   *
   * @return time in milliseconds at which the test ended
   */
  public long launch() {

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("in", BpelXmlUtil.createElementWithContent("Dupont"));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";

    CallResult callResult = call(parts, portTypeQName, operationName);
    long endTime = callResult.getEndTime();

    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = callResult.getMessageCarrier().getMessage();
      if (reply == null) {
        throw new OrchestraException("reply is null");
      }
      Element element = reply.getPartValue("out");
      if (element == null) {
        throw new OrchestraException("reply.out is null");
      }
      String result = element.getTextContent();
      if (result == null) {
        throw new OrchestraException("reply.out.textContent is null");
      }
      String expected = "flowResultABCDE_AE_BF";
      if (!result.equals(expected)) {
        throw new OrchestraException("reply.out.textContent is not " + expected
            + ", it is : " + result);
      }
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  public void testFlow() {
    deploy();
    launch();
    undeploy();
  }

  public String getAlias() {
    return "class.flow";
  }
}
