package org.ow2.orchestra.test.perf.loanApproval;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.test.WSRepository;
import org.ow2.orchestra.test.perf.AbstractPerfTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class LoanApprovalTest extends AbstractPerfTestCase {

  public LoanApprovalTest() {
    super("http://orchestra.ow2.org/loanApproval/loanService", "loanService");
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
    WSRepository.addWS(RiskAssessmentWS.PORTTYPE_QNAME, new RiskAssessmentWS());
    WSRepository.addWS(ApprovalWS.PORTTYPE_QNAME, new ApprovalWS());
  }

  public long launch() {
    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("firstName", BpelXmlUtil.createElementWithContent("Charles"));
    parts.put("name", BpelXmlUtil.createElementWithContent("Dupont"));
    int amount = BpelUtil.getRandomInt(0, 60000);
    parts.put("amount", BpelXmlUtil.createElementWithContent(Integer
        .toString(amount)));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "request";
    CallResult callResult = call(parts, portTypeQName, operationName);
    long endTime = callResult.getEndTime();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = callResult.getMessageCarrier().getMessage();

      if (reply == null) {
        throw new OrchestraException("reply is null");
      }
      Element element = reply.getPartValue("accept");
      if (element == null) {
        throw new OrchestraException("reply.accept is null");
      }
      String accept = element.getTextContent();
      if (amount >= 30000 && accept.equals("yes")) {
        throw new OrchestraException(
            "reply.accept.textContent must be no (amount=" + amount
                + "), it is : " + accept);
      } else if (amount < 30000 && accept.equals("no")) {
        throw new OrchestraException(
            "reply.accept.textContent must be yes (amount=" + amount
                + "), it is : " + accept);
      }
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  public static class ApprovalWS {

    private static final String NAMESPACE = "http://orchestra.ow2.org/loanApproval/approval";

    public static final QName PORTTYPE_QNAME = new QName(NAMESPACE,
        "approvalPT");

    public Message approve(VariableRuntime variableRuntime) {
      Object varValue = variableRuntime.getValue();
      if (!(varValue instanceof Message)) {
        throw new OrchestraException("variableRuntime value is not a message");
      }
      Message creditInformationRequest = (Message) varValue;
      String name = creditInformationRequest.getPartValue("name")
          .getTextContent();
      String amountSt = creditInformationRequest.getPartValue("amount")
          .getTextContent();
      int amount = new Integer(amountSt);

      String accept = "no";
      if ("Dupont".equals(name) || "McDonalds".equals(name)) {
        if (amount < 30000) {
          accept = "yes";
        }
      } else if ("Kent".equals(name) || "Martin".equals(name)) {
        if (amount < 10000) {
          accept = "yes";
        }
      } else if (amount < 50000) {
        accept = "yes";
      }

      Element element = BpelXmlUtil.createElementWithContent(accept);
      HashMap<String, Element> parts = new HashMap<String, Element>();
      parts.put("accept", element);
      return new Message(parts);
    }
  }

  public static class RiskAssessmentWS {

    private static final String NAMESPACE = "http://orchestra.ow2.org/loanApproval/riskAssessment";

    public static final QName PORTTYPE_QNAME = new QName(NAMESPACE,
        "riskAssessmentPT");

    public Message check(VariableRuntime variableRuntime) {
      Object varValue = variableRuntime.getValue();
      if (!(varValue instanceof Message)) {
        throw new OrchestraException("variableRuntime value is not a message");
      }
      Message creditInformationRequest = (Message) varValue;
      String name = creditInformationRequest.getPartValue("name")
          .getTextContent();

      String level = "low";
      if ("Dupont".equals(name) || "McDonalds".equals(name)) {
        level = "medium";
      } else if ("Kent".equals(name) || "Martin".equals(name)) {
        level = "high";
      }
      Element outputElement = BpelXmlUtil.createElementWithContent(level);
      HashMap<String, Element> parts = new HashMap<String, Element>();
      parts.put("level", outputElement);
      return new Message(parts);
    }
  }

  public void undeploy() {
    super.undeploy();
    WSRepository.removeWS(RiskAssessmentWS.PORTTYPE_QNAME);
    WSRepository.removeWS(ApprovalWS.PORTTYPE_QNAME);
  }

  public void testAddition() {
    deploy();
    launch();
    undeploy();
  }

  public String getAlias() {
    return "class.loan";
  }

}
