/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.standardElements.transitionCondition;

import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.jbpm.tx.Transaction;
import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.exception.faults.JoinFailure;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.MutexRepository;
import org.ow2.orchestra.services.itf.Repository;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class TransitionConditionTest extends BpelTestCase {


  /**
   * Default constructor.
   */
  public TransitionConditionTest() {
    super("http://example.com/transitionCondition", "transitionCondition");
  }

  public void testTransitionCondition() {
    deploy();
    launch();
    launchBadRequest("badRequest1.xml");
    launchBadRequest("badRequest2.xml");
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this.getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL request = this.getClass().getResource("request.xml");
    assertExists(request);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("p", BpelUtil.getDocumentFromURL(request).getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName() + "PT");
    String operationName = "submit";

    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment =  getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getEvents());
      assertEquals(2, callResult.getEvents().size());

      Iterator<String> i = callResult.getEvents().iterator();
      assertEquals("B", i.next());
      assertEquals("A", i.next());
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  public long launchBadRequest(String resource) {
    URL request = this.getClass().getResource(resource);
    assertExists(request);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("p", BpelUtil.getDocumentFromURL(request).getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName() + "PT");
    String operationName = "submit";

    try {
      call(parts, portTypeQName, operationName);
      fail("Expected join failure exception.");
    } catch (JoinFailure e) {
      // nothing to do
    }
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Repository repository = environment.get(Repository.class);
      List<BpelExecution> instances = repository.getInstances(new QName(getProcessNamespace(), getProcessName()));
      assertEquals(1, instances.size());
      BpelExecution instance = instances.get(0);
      repository.removeInstance(instance);
      MutexRepository.removeInstanceMutex(instance.getUuid());
      
    } catch (Exception e) {
      if (environment != null) {
        Transaction t = environment.getTransaction();
        if (t != null) {
          t.setRollbackOnly();
        }
      }
      throw new OrchestraException(e);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return System.currentTimeMillis();
  }

}


