/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.var.fromAii;

import java.net.URL;
import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class FromAiiTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public FromAiiTest() {
    super("http://example.com/fromAii", "fromAii");
  }

  public void testFromAii() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL baseProduct = this.getClass().getResource("baseProduct.xml");
    assertExists(baseProduct);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("p", BpelUtil.getDocumentFromURL(baseProduct)
        .getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());
      assertEquals(13, callResult.getVariables().size());

      // aiiToAii
      assertProductAmount(getVariableValue(callResult, "productAmount1"));
      assertProductAmount(getVariableValue(callResult, "productAmount2"));
      assertProductReference(getVariableValue(callResult, "productReference1"));
      assertProductReference(getVariableValue(callResult, "productReference2"));
      assertBillingDetailVar(getVariableValue(callResult, "billingDetailVar"));

      // aiiToEii
      assertProductQuantity(getVariableValue(callResult, "productQuantity1"));
      assertProductQuantity(getVariableValue(callResult, "productQuantity2"));
      assertProductName(getVariableValue(callResult, "productName1"));
      assertProductName(getVariableValue(callResult, "productName2"));
      assertBillingDetailVar2(getVariableValue(callResult, "billingDetailVar2"));

      // aiiToTii
      assertProductQuantity(getVariableValue(callResult, "productQuantity3"));
      assertProductName(getVariableValue(callResult, "productName3"));
      assertBillingAmountVar(getVariableValue(callResult, "billingAmountVar"));
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertProductAmount(Object o) {
    Element root = assertTopElementNS(o, "product");
    assertEquals("123", root.getAttribute("amount"));
    assertEquals("", root.getAttribute("reference"));
    assertNull(root.getFirstChild());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertProductReference(Object o) {
    Element root = assertTopElementNS(o, "product");
    assertEquals("", root.getAttribute("amount"));
    assertEquals("chips", root.getAttribute("reference"));
    assertNull(root.getFirstChild());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertProductQuantity(Object o) {
    Element root = assertTopElementNS(o, "product");
    assertEquals("", root.getAttribute("amount"));
    assertEquals("", root.getAttribute("reference"));
    assertElementNSContentAndNoChild(root, "quantity", "123");
    assertElementNSContentAndNoChild(root, "name", "");
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertProductName(Object o) {
    Element root = assertTopElementNS(o, "product");
    assertEquals("", root.getAttribute("amount"));
    assertEquals("", root.getAttribute("reference"));
    assertElementNSContentAndNoChild(root, "quantity", "");
    assertElementNSContentAndNoChild(root, "name", "chips");
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertBillingDetailVar(Object o) {
    Element root = assertTopElementNS(o, "billingDetail");
    assertEquals("2299", root.getAttribute("amt"));
    assertNoElementChild(root);
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertBillingDetailVar2(Object o) {
    Element root = assertTopElementNS(o, "billingDetail");
    assertEquals("8675309", root.getAttribute("id"));
    Element billingAmount = assertElementNSContentAndNoChild(root,
        "billingAmount", "3399");
    assertEquals("F00B2R", billingAmount.getAttribute("code"));
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertBillingAmountVar(Object o) {
    Element root = assertTopElementNS(o, "billingAmount");
    assertEquals("4499", root.getTextContent());
    assertNoElementChild(root);
  }

}
