/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.var.fromEii;

import java.net.URL;
import java.util.HashMap;
import java.util.List;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class FromEiiTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public FromEiiTest() {
    super("http://example.com/fromEii", "fromEii");
  }

  public void testFromEii() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL request = this.getClass().getResource("request.xml");
    assertExists(request);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("p", BpelUtil.getDocumentFromURL(request).getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());

      // eiiToAii
      assertProductAmount(getVariableValue(callResult, "productAmount1"));
      assertProductReference(getVariableValue(callResult, "productReference1"));
      assertApprovalNotice2Var(getVariableValue(callResult,
          "approvalNotice2Var"));

      // eiiToEii
      assertPoHeaderVar2(getVariableValue(callResult, "poHeaderVar2"));
      // assertPoHeaderVar4(getVariableValue(callResult, "poHeaderVar4"));

      // eiiToTii
      assertApprovalNotice3Var(getVariableValue(callResult,
          "approvalNotice3Var"));
      assertApprovalNotice4Var(getVariableValue(callResult,
          "approvalNotice4Var"));
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertProductAmount(Object o) {
    Element root = assertTopElementNS(o, "product");
    assertEquals("52", root.getAttribute("amount"));
    assertEquals("", root.getAttribute("reference"));
    assertNull(root.getFirstChild());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertProductReference(Object o) {
    Element root = assertTopElementNS(o, "product");
    assertEquals("", root.getAttribute("amount"));
    assertEquals("myProduct", root.getAttribute("reference"));
    assertNull(root.getFirstChild());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertPoHeaderVar2(Object o) {
    Element root = assertTopElementNS(o, "poHeader");
    List<Element> children = XmlUtil.elements(root, getProcessNamespace(),
        "billingAddr");
    assertEquals(1, children.size());
    Element billingAddr = (Element) children.get(0);
    assertEquals("true", billingAddr.getAttribute("verified"));
    assertElementNSContentAndNoChild(billingAddr, "street", "123 Main Street");
    assertElementNSContentAndNoChild(billingAddr, "city", "SomeWhere City");
    assertElementNSContentAndNoChild(billingAddr, "country", "UK");
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertApprovalNotice2Var(Object o) {
    Element root = assertTopElementNS(o, "approvalNotice");
    assertNoElementChild(root);
    assertEquals("4500", root.getAttribute("amt"));
    assertEquals("", root.getTextContent());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertApprovalNotice3Var(Object o) {
    Element root = assertTopElementNS(o, "approvalNotice");
    assertNoElementChild(root);
    assertEquals("4500", root.getTextContent());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertApprovalNotice4Var(Object o) {
    Element root = assertTopElementNS(o, "approvalNotice");
    assertNoElementChild(root);
    assertEquals("AXR", root.getAttribute("code"));
    assertEquals("4500", root.getTextContent());
  }

}
