/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.var.fromTii;

import java.net.URL;
import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class FromTiiTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public FromTiiTest() {
    super("http://example.com/fromTii", "fromTii");
  }

  public void testFromTii() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL request = this.getClass().getResource("request.xml");
    assertExists(request);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("p", BpelUtil.getDocumentFromURL(request).getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());
      assertEquals(3, callResult.getVariables().size());

      // aiiToAii
      assertShippingAddressVar1(getVariableValue(callResult,
          "shippingAddressVar1"));

      // aiiToEii
      assertShippingAddressVar2(getVariableValue(callResult,
          "shippingAddressVar2"));

      // aiiToTii
      assertShippingPostalCodeVar(getVariableValue(callResult,
          "shippingPostalCodeVar"));
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertShippingAddressVar1(Object o) {
    Element root = assertTopElementNS(o, "shippingAddress");
    assertEquals("94304", root.getAttribute("postCode"));
    assertNoElementChild(root);
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertShippingAddressVar2(Object o) {
    Element root = assertTopElementNS(o, "shippingAddress");
    assertEquals("9035768", root.getAttribute("id"));
    Element postlCode = assertElementNSContentAndNoChild(root, "postalCode",
        "94107");
    assertNoElementChild(postlCode);
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertShippingPostalCodeVar(Object o) {
    Element root = assertTopElementNS(o, "shippingPostalCode");
    assertEquals("95110", root.getTextContent());
    assertNoElementChild(root);
  }

}
