/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.var.literal;

import java.net.URL;
import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class LiteralTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public LiteralTest() {
    super("http://example.com/literal", "literal");
  }

  public void testLiteral() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL baseProduct = this.getClass().getResource("baseProduct.xml");
    assertExists(baseProduct);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("p", BpelUtil.getDocumentFromURL(baseProduct)
        .getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());
      assertEquals(8, callResult.getVariables().size());

      // strings
      assertHelloWorld(getVariableValue(callResult, "st1"));
      assertHelloWorld(getVariableValue(callResult, "st2"));

      // myElement
      assertmyElement(getVariableValue(callResult, "element1"));
      assertmyElement(getVariableValue(callResult, "element2"));
      assertmyElement(getVariableValue(callResult, "element3"));
      assertmyElement(getVariableValue(callResult, "element4"));
      assertmyElement(getVariableValue(callResult, "element5"));
      assertmyElement(getVariableValue(callResult, "element6"));
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertHelloWorld(Object o) {
    assertString(o, "Hello World!!");
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertmyElement(Object o) {
    Element root = assertTopElementNS(o, "myElement");
    assertElementNSContentAndNoChild(root, "element1.1.1", "HelloWorld");
    assertElementNSContentAndNoChild(root, "element1.2", "123");
  }

}
