/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.var.message;

import java.net.URL;
import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class MessageTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public MessageTest() {
    super("http://example.com/message", "message");
  }

  public void testMessage() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL baseProduct = this.getClass().getResource("baseProduct.xml");
    assertExists(baseProduct);
    URL baseElement = this.getClass().getResource("baseElement.xml");
    assertExists(baseElement);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("product", BpelUtil.getDocumentFromURL(baseProduct)
        .getDocumentElement());
    parts.put("element", BpelUtil.getDocumentFromURL(baseElement)
        .getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());
      assertEquals(6, callResult.getVariables().size());

      // fromMessageToMessage
      assertMessage(getVariableValue(callResult, "request2"));

      // fromMessagePartToElement
      assertProduct(getVariableValue(callResult, "productPart"));
      assertmyElement(getVariableValue(callResult, "elementPart"));
      assertProduct(getVariableValue(callResult, "productPart2"));
      assertmyElement(getVariableValue(callResult, "elementPart2"));

      // fromElementToMessagePart
      assertMessage(getVariableValue(callResult, "request3"));
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertMessage(Object o) {
    assertTrue("o is not an instance of Message", o instanceof Message);
    assertEquals(2, ((Message) o).getParts().size());
    Object productPart = ((Message) o).getPartValue("product");
    assertProduct(productPart);
    Object elementPart = ((Message) o).getPartValue("element");
    assertmyElement(elementPart);
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertProduct(Object o) {
    Element root = assertTopElementNS(o, "product");
    assertEquals("123", root.getAttribute("amount"));
    assertEquals("chips", root.getAttribute("reference"));
    assertElementNSContentAndNoChild(root, "name", "myProduct");
    assertElementNSContentAndNoChild(root, "quantity", "52");
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertmyElement(Object o) {
    Element root = assertTopElementNS(o, "myElement");
    assertElementNSContentAndNoChild(root, "element1.1.1", "HelloWorld");
    assertElementNSContentAndNoChild(root, "element1.2", "123");
  }

}
