/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.var.property;

import java.net.URL;
import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class PropertyTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public PropertyTest() {
    super("http://example.com/property", "property");
  }

  public void testProperty() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL creditCardNumber = this.getClass().getResource("creditCardNumber.xml");
    assertExists(creditCardNumber);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("creditCardNumber", BpelUtil.getDocumentFromURL(creditCardNumber)
        .getDocumentElement());
    parts.put("shippingCosts", BpelXmlUtil.createElementWithContent("12345"));
    parts.put("auctionId", BpelXmlUtil.createElementWithContent("54321"));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());
      assertEquals(3, callResult.getVariables().size());

      // auctionIdOnMessage
      assertRequest(getVariableValue(callResult, "request"));
      assertRequest2(getVariableValue(callResult, "request2"));

      // auctionIdOnElement
      assertString(getVariableValue(callResult, "st"));
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertRequest(Object o) {
    assertTrue("request is not an instance of Message", o instanceof Message);
    assertEquals(3, ((Message) o).getParts().size());
    Element shippingCostsPart = ((Message) o).getPartValue("shippingCosts");
    assertEquals("54321", shippingCostsPart.getTextContent());
    Element creditCardNumber = ((Message) o).getPartValue("creditCardNumber");
    assertNotNull(creditCardNumber);
    Element auctionId = ((Message) o).getPartValue("auctionId");
    assertNotNull(auctionId);
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertRequest2(Object o) {
    assertTrue("request is not an instance of Message", o instanceof Message);
    assertEquals(3, ((Message) o).getParts().size());
    Element shippingCostsPart = ((Message) o).getPartValue("shippingCosts");
    assertEquals("54321", shippingCostsPart.getTextContent());
    //Element creditCardNumber = ((Message) o).getPartValue("creditCardNumber");
    // message was initialized so part is not null but contains a default
    // element using part element or type QName
    // assertNull(creditCardNumber);
    //Element auctionId = ((Message) o).getPartValue("auctionId");
    // message was initialized so part is not null but contains a default
    // element using part element or type QName
    // assertNull(auctionId);
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertString(Object o) {
    assertTrue("st is not an instance of Element", o instanceof Element);
    assertNotNull((Element) o);
    assertEquals("123", ((Element) o).getTextContent());
  }

}
