/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.var.query;

import java.net.URL;
import java.util.HashMap;
import java.util.List;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.jbpm.xml.XmlUtil;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author Charles Souillard
 */
public class QueryTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public QueryTest() {
    super("http://example.com/query", "query");
  }

  public void testQuery() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL bigProduct = this.getClass().getResource("bigProduct.xml");
    assertExists(bigProduct);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts
        .put("p", BpelUtil.getDocumentFromURL(bigProduct).getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      assertNotNull(callResult.getVariables());
      assertEquals(8, callResult.getVariables().size());

      // from var query
      assertString(getVariableValue(callResult, "st1"));
      assertElement(getVariableValue(callResult, "element1"));
      assertQuantity(getVariableValue(callResult, "quantity1"));

      // to var query
      assertToQuery(getVariableValue(callResult, "toQuery"));

      // from var part query
      assertString(getVariableValue(callResult, "st2"));
      assertElement(getVariableValue(callResult, "element2"));
      assertQuantity(getVariableValue(callResult, "quantity2"));

      // to var part query
      assertRequest(getVariableValue(callResult, "request"));
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertString(Object o) {
    assertString(o, "myAttrValue");
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertElement(Object o) {
    Element element1 = assertTopElementNS(o, "element1.1.1");
    assertEquals("HelloWorld", element1.getTextContent());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertQuantity(Object o) {
    Element quantity = assertTopElementNS(o, "quantity");
    assertEquals("52", quantity.getTextContent());
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertToQuery(Object o) {
    Element bigProduct = assertTopElementNS(o, "bigProduct");
    assertEquals("", bigProduct.getAttribute("amount"));
    assertEquals("", bigProduct.getAttribute("reference"));
    assertElementNSContentAndNoChild(bigProduct, "name", "");
    assertElementNSContentAndNoChild(bigProduct, "quantity", "486");
    List<Element> myElementChildren = XmlUtil.elements(bigProduct,
        getProcessNamespace(), "myElement");
    assertEquals(1, myElementChildren.size());
    Node myElementNode = myElementChildren.get(0);
    assertTrue(myElementNode instanceof Element);
    Element myElement = (Element) myElementNode;
    assertElementNSContentAndNoChild(myElement, "element1.1.1", "fromElement1");
    assertElementNSContentAndNoChild(myElement, "element1.2", "");
  }

  /**
   * Result assertion.
   *
   * @param o
   *            object to assert.
   */
  private void assertRequest(Object o) {
    assertTrue("request is not an instance of Message", o instanceof Message);
    Element bigProductElement = ((Message) o).getPartValue("p");

    Element bigProduct = assertTopElementNS(bigProductElement, "bigProduct");
    assertEquals("123", bigProduct.getAttribute("amount"));
    assertEquals("chips", bigProduct.getAttribute("reference"));
    assertElementNSContentAndNoChild(bigProduct, "name", "myProduct");
    assertElementNSContentAndNoChild(bigProduct, "quantity", "486");
    List<Element> myElementChildren = XmlUtil.elements(bigProduct,
        getProcessNamespace(), "myElement");
    assertEquals(1, myElementChildren.size());
    Node myElementNode = myElementChildren.get(0);
    assertTrue(myElementNode instanceof Element);
    Element myElement = (Element) myElementNode;
    assertElementNSContentAndNoChild(myElement, "element1.1.1", "fromElement1");
    assertElementNSContentAndNoChild(myElement, "element1.2", "123");
  }

}
