/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.wsutils;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class PersonWS {

  private static final String NAMESPACE = "http://orchestra.ow2.org/personWS";

  public static final QName PORTTYPE_QNAME = new QName(NAMESPACE, "personWSPT");

  public Message createPerson(VariableRuntime variableRuntime) {
    Object varValue = variableRuntime.getValue();
    if (!(varValue instanceof Message)) {
      throw new OrchestraException("variableRuntime value is not a message");
    }

    Message createRequest = (Message) varValue;
    String firstName = createRequest.getPartValue("firstName").getTextContent();
    String lastName = createRequest.getPartValue("lastName").getTextContent();
    String age = createRequest.getPartValue("age").getTextContent();
    String street = createRequest.getPartValue("street").getTextContent();
    String zip = createRequest.getPartValue("zip").getTextContent();
    String city = createRequest.getPartValue("city").getTextContent();

    Element personRootElement = BpelUtil.getDocumentWithOneElement(new QName(
        NAMESPACE, "createPerson"));
    Document doc = personRootElement.getOwnerDocument();

    Element firstNameEl = doc.createElementNS(NAMESPACE, "firstName");
    firstNameEl.appendChild(doc.createTextNode(firstName));
    personRootElement.appendChild(firstNameEl);

    Element lastNameEl = doc.createElementNS(NAMESPACE, "lastName");
    lastNameEl.appendChild(doc.createTextNode(lastName));
    personRootElement.appendChild(lastNameEl);

    Element ageEl = doc.createElementNS(NAMESPACE, "age");
    ageEl.appendChild(doc.createTextNode(age));
    personRootElement.appendChild(ageEl);

    Element addressEl = doc.createElementNS(NAMESPACE, "address");
    personRootElement.appendChild(addressEl);

    Element streetEl = doc.createElementNS(NAMESPACE, "street");
    streetEl.appendChild(doc.createTextNode(street));
    addressEl.appendChild(streetEl);

    Element zipEl = doc.createElementNS(NAMESPACE, "zip");
    zipEl.appendChild(doc.createTextNode(zip));
    addressEl.appendChild(zipEl);

    Element cityEl = doc.createElementNS(NAMESPACE, "city");
    cityEl.appendChild(doc.createTextNode(city));
    addressEl.appendChild(cityEl);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("person", personRootElement);
    return new Message(parts);
  }

  public Message addPerson(VariableRuntime variableRuntime) {
    Object varValue = variableRuntime.getValue();
    if (!(varValue instanceof Message)) {
      throw new OrchestraException("variableRuntime value is not a message");
    }

    Message addRequest = (Message) varValue;
    Element personToAdd = addRequest.getPartValue("personToAdd");
    Element personsList = addRequest.getPartValue("personsList");

    Document doc = personsList.getOwnerDocument();

    Node importedPerson = doc.importNode(personToAdd, true);
    personsList.appendChild(importedPerson);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("personsList", personsList);
    return new Message(parts);
  }

  public Message returnReceived(VariableRuntime variableRuntime) {
    Object varValue = variableRuntime.getValue();
    if (!(varValue instanceof Message)) {
      throw new OrchestraException("variableRuntime value is not a message");
    }
    Message request = (Message) varValue;
    return request;
  }
}
