/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.wsutils;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.ow2.orchestra.exception.OrchestraException;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class StringWS {

  private String st;

  private static final String NAMESPACE = "http://orchestra.ow2.org/stringWS";

  public static final QName PORTTYPE_QNAME = new QName(NAMESPACE, "stringWSPT");

  public StringWS(String st) {
    this.st = st;
  }

  public Message suffix(VariableRuntime variableRuntime) {
    Object varValue = variableRuntime.getValue();
    if (!(varValue instanceof Message)) {
      throw new OrchestraException("variableRuntime value is not a message");
    }
    Message suffixRequest = (Message) varValue;
    Element suffixElement = suffixRequest.getPartValue("input");
    if (!suffixElement.getLocalName().equals("suffix")) {
      throw new OrchestraException(
          "input part root element local name is not 'suffix'");
    }
    if (!suffixElement.getNamespaceURI().equals(NAMESPACE)) {
      throw new OrchestraException(
          "input part root element namespaceURI is not '" + NAMESPACE + "'");
    }
    if (!(suffixElement.getFirstChild() instanceof Text)) {
      throw new OrchestraException("input part first child is not a text");
    }

    Text text = (Text) suffixElement.getFirstChild();
    String content = text.getTextContent();

    Element outputRootElement = BpelUtil.getDocumentWithOneElement(new QName(
        NAMESPACE, "suffix"));
    Text textContent = outputRootElement.getOwnerDocument().createTextNode(
        content + st);
    outputRootElement.appendChild(textContent);

    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("output", outputRootElement);
    return new Message(parts);
  }

}
