/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.xpath.coreFunctions;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class CoreFunctionsTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public CoreFunctionsTest() {
    super("http://example.com/coreFunctions", "coreFunctions");
  }

  public void testCoreFunctions() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("in", BpelXmlUtil.createElementWithContent("Dupont"));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Object variable3 = getVariableValue(callResult, "variable3");
      assertNotNull(variable3);
      assertTrue("variable1 is not an Element", variable3 instanceof Element);
      assertEquals("Hello World!!", ((Element) variable3).getTextContent());

      Object variable6 = getVariableValue(callResult, "variable6");
      assertNotNull(variable6);
      assertTrue("variable6 is not an Element", variable6 instanceof Element);
      assertEquals("10", ((Element) variable6).getTextContent());

      Object variable8 = getVariableValue(callResult, "variable8");
      assertNotNull(variable8);
      assertTrue("variable8 is not an Element", variable8 instanceof Element);
      assertEquals("false", ((Element) variable8).getTextContent());

      Object variable9 = getVariableValue(callResult, "variable9");
      assertNotNull(variable9);
      assertTrue("variable9 is not an Element", variable9 instanceof Element);
      assertEquals("4", ((Element) variable9).getTextContent());

      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

}
