/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.xpath.doXslTransformationFunction;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class DoXslTransformFunctionTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public DoXslTransformFunctionTest() {
    super("http://example.com/doXslTransformationFunction",
        "doXslTransformationFunction");
  }

  public void testXslTransform() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    HashMap<String, Element> parts = new HashMap<String, Element>();
    parts.put("st", BpelXmlUtil.createElementWithContent("Begin"));

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Message reply = callResult.getMessageCarrier().getMessage();

      assertEquals("Empire Burlesque", reply.getPartValue("st1")
          .getTextContent());
      assertEquals("9", reply.getPartValue("st2").getTextContent());
      assertEquals("Dolly Parton", reply.getPartValue("st3").getTextContent());
      assertEquals("50", reply.getPartValue("st4").getTextContent());
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

}
