/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.test.xpath.getVariablePropertyFunction;

import java.net.URL;
import java.util.HashMap;

import javax.xml.namespace.QName;

import org.jbpm.env.Environment;
import org.ow2.orchestra.test.BpelTestCase;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.var.Message;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */
public class GetVariablePropertyFunctionTest extends BpelTestCase {

  /**
   * Default constructor.
   */
  public GetVariablePropertyFunctionTest() {
    super("http://example.com/getVariablePropertyFunction",
        "getVariablePropertyFunction");
  }

  public void testGetVariablePropertyFunction() {
    deploy();
    launch();
    undeploy();
  }

  public void deploy() {
    deploy(this.getClass().getResource(getProcessName() + ".bpel"), this
        .getClass().getResource(getProcessName() + ".wsdl"));
  }

  public long launch() {
    URL variable1Part1 = this.getClass().getResource("variable1_part1.xml");
    assertExists(variable1Part1);
    URL variable1Part2 = this.getClass().getResource("variable1_part2.xml");
    assertExists(variable1Part2);
    URL variable1Part3 = this.getClass().getResource("variable1_part3.xml");
    assertExists(variable1Part3);

    HashMap<String, Element> parts = new HashMap<String, Element>();

    parts.put("creditCardNumber", BpelUtil.getDocumentFromURL(variable1Part1)
        .getDocumentElement());
    parts.put("shippingCosts", BpelUtil.getDocumentFromURL(variable1Part2)
        .getDocumentElement());
    parts.put("auctionId", BpelUtil.getDocumentFromURL(variable1Part3)
        .getDocumentElement());

    QName portTypeQName = new QName(getProcessNamespace(), getProcessName()
        + "PT");
    String operationName = "submit";
    CallResult callResult = call(parts, portTypeQName, operationName);

    long endTime = System.currentTimeMillis();
    Environment environment = null;
    try {
      environment = getEnvironmentFactory().openEnvironment();
      Object variable1 = getVariableValue(callResult, "variable1");
      assertNotNull(variable1);
      assertTrue("variable1 is not a message", variable1 instanceof Message);
      assertEquals("54321", ((Message) variable1).getPartValue("shippingCosts")
          .getTextContent());

      Object variable3 = getVariableValue(callResult, "variable3");
      assertNotNull(variable3);
      assertTrue("variable3 is not an Element", variable3 instanceof Element);
      assertEquals("123", ((Element) variable3).getTextContent());

      Object variable4 = getVariableValue(callResult, "variable4");
      assertNotNull(variable4);
      assertTrue("variable4 is not a message", variable4 instanceof Message);
      assertEquals("123", ((Message) variable4).getPartValue("shippingCosts")
          .getTextContent());
      deleteInstance(callResult);
    } finally {
      if (environment != null) {
        environment.close();
      }
    }
    return endTime;
  }

}
